# Volatility
#
# Authors:
# Mike Auty <mike.auty@gmail.com>
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License Version 2 as
# published by the Free Software Foundation.  You may not use, modify or
# distribute this program under any other version of the GNU General
# Public License.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

import re
import sre_constants
import struct
import volatility.plugins.procdump as procdump
import volatility.win32 as win32
import volatility.obj as obj
import volatility.utils as utils
import volatility.addrspace as addrspace
import volatility.debug as debug
import volatility.exceptions as exceptions

MAX_STRING_BYTES = 260

ver_types = {
'_IMAGE_RESOURCE_DIRECTORY' : [ 0x12, {
  'Characteristics' : [ 0x0, ['unsigned long']],
  'Timestamp' : [ 0x4, ['unsigned long']],
  'MajorVersion': [ 0x8, ['unsigned short']],
  'Minorversion': [ 0xa, ['unsigned short']],
  'NamedEntriesCount': [ 0xc, ['unsigned short']],
  'IdEntriesCount': [0xe, ['unsigned short']],
  'Entries': [0x10, ['array', lambda x: x.NamedEntriesCount + x.IdEntriesCount, ['_IMAGE_RESOURCE_DIRECTORY_ENTRY']]],
} ],
'_IMAGE_RESOURCE_DIRECTORY_ENTRY': [0x8, {
  'Name' : [ 0x0, ['unsigned long']],
  'DataOffset' : [ 0x4, ['unsigned long']],
} ],
'_IMAGE_RESOURCE_DATA_ENTRY' : [0x10, {
  'DataOffset' : [0x0, ['unsigned long']],
  'Size' : [0x4, ['unsigned long']],
  'CodePage' : [0x8, ['unsigned long']],
  'Reserved' : [0xc, ['unsigned long']],
} ],
'_IMAGE_RESOURCE_DIR_STRING_U' : [0x4, {
  'Length': [0x0, ['unsigned short']],
  'Value' : [0x2, ['array', lambda x: x.Length, ['unsigned short']]],
} ],
'_VS_VERSION_INFO' : [0x26, {
  'Length': [0x0, ['unsigned short']],
  'ValueLength': [0x2, ['unsigned short']],
  'Type': [0x4, ['unsigned short']],
  'Key': [0x6, ['array', len("VS_VERSION_INFO "), ['unsigned short']]],
  'FileInfo': [lambda x: (((x.Key.obj_offset + x.Key.size() + 3) / 4) * 4), ['_VS_FIXEDFILEINFO']],
} ],
'VerStruct' : [0x26, {
  'Length': [0x0, ['unsigned short']],
  'ValueLength': [0x2, ['unsigned short']],
  'Type': [0x4, ['unsigned short']],
  'Key': [0x6, ['array', MAX_STRING_BYTES, ['unsigned short']]],
} ],
'_VS_FIXEDFILEINFO': [0x34, {
  'Signature': [0x0, ['unsigned long']],
  'StructVer': [0x4, ['unsigned long']],
  'FileVerMS': [0x8, ['unsigned long']],
  'FileVerLS': [0xC, ['unsigned long']],
  'ProdVerMS': [0x10, ['unsigned long']],
  'ProdVerLS': [0x14, ['unsigned long']],
  'FileFlagsMask': [0x18, ['unsigned long']],
  'FileFlags': [0x1C, ['unsigned long']],
  'FileOS': [0x20, ['Enumeration', {'choices': {
    0x0: 'Unknown',
    0x10000: 'DOS',
    0x20000: 'OS/2 16-bit',
    0x30000: 'OS/2 32-bit',
    0x40000: 'Windows NT',
    0x1: 'Windows 16-bit',
    0x2: 'Presentation Manager 16-bit',
    0x3: 'Presentation Manager 32-bit',
    0x4: 'Windows 32-bit',
    0x10001: 'Windows 16-bit running on DOS',
    0x10004: 'Windows 32-bit running on DOS',
    0x20002: 'Presentation Manager running on OS/2 (16-bit)',
    0x30003: 'Presentation Manager running on OS/2 (32-bit)',
    0x40004: 'Windows NT',
                                                  }} ]],
  'FileType': [0x24, ['Enumeration', {'choices': {
    0x0: 'Unknown',
    0x1: 'Application',
    0x2: 'Dynamic Link Library',
    0x3: 'Driver',
    0x4: 'Font',
    0x5: 'Virtual Device',
    0x7: 'Static Library',
                                                  }} ]],
  'FileSubType': [0x28, ['unsigned long']],
  'FileDate': [0x2C, ['WinTimeStamp']],
} ],
}

class VerStruct(obj.CType):
    """Generic Version Structure"""

    def _determine_key(self, findend = False):
        """Determines the string value for or end location of the key"""
        if self.Key != None:
            name = None
            for n in self.Key:
                if n == None:
                    return n
                # If the letter's valid, then deal with it
                if n == 0:
                    if findend:
                        return n.obj_offset + n.size()
                    name = self.obj_vm.read(self.Key.obj_offset, n.obj_offset - self.Key.obj_offset).decode("utf16", "ignore").encode("ascii", 'backslashreplace')
                    break
            return name
        return self.Key

    def get_key(self):
        """Returns the VerStruct Name"""
        return self._determine_key()

    def offset_pad(self, offset):
        """Pads an offset to a 32-bit alignment"""
        return (((offset + 3) / 4) * 4)

    def get_children(self):
        """Returns the available children"""
        offset = self.offset_pad(self._determine_key(True))
        if self.ValueLength > 0:
            # Nasty hardcoding unicode (length*2) length in here, 
            # but what else can we do?
            return self.obj_vm.read(offset, self.ValueLength * 2)
        else:
            return self._recurse_children(offset)

    def _recurse_children(self, offset):
        """Recurses thorugh the available children"""
        while offset < self.obj_offset + self.Length:
            item = obj.Object("VerStruct", offset = offset, vm = self.obj_vm, parent = self)
            if item.Length < 1 or item.get_key() == None:
                raise StopIteration("Could not recover a key for a child at offset {0}".format(item.obj_offset))
            yield item.get_key(), item.get_children()
            offset = self.offset_pad(offset + item.Length)
        raise StopIteration("No children")

class _VS_VERSION_INFO(VerStruct):
    """Version Information"""

    def get_children(self):
        """Recurses through the children of a Version Info records"""
        offset = self.offset_pad(self.FileInfo.obj_offset + self.ValueLength)
        return self._recurse_children(offset)

class _VS_FIXEDFILEINFO(obj.CType):
    """Fixed (language and codepage independent) information"""

    def file_version(self):
        """Returns the file version"""
        return self.get_version(self.FileVerMS) + "." + self.get_version(self.FileVerLS)

    def product_version(self):
        """Returns the product version"""
        return self.get_version(self.ProdVerMS) + "." + self.get_version(self.ProdVerLS)

    def get_version(self, value):
        """Returns a version in four parts"""
        version = []
        for i in range(2):
            version = [(value >> (i * 16)) & 0xFFFF] + version
        return '.'.join([str(x) for x in version])

    def file_type(self):
        """Returns the type of the file"""
        ftype = str(self.FileType)
        choices = None
        if self.FileType == 'Driver':
            choices = {
                       0x0: 'Unknown',
                       0x1: 'Printer',
                       0x2: 'Keyboard',
                       0x3: 'Language',
                       0x4: 'Display',
                       0x5: 'Mouse',
                       0x6: 'Network',
                       0x7: 'System',
                       0x8: 'Installable',
                       0x9: 'Sound',
                       0xA: 'Comms',
                       0xB: 'Input Method',
                       0xC: 'Versioned Printer',
                       }
        elif self.FileType == 'Font':
            choices = {
                       0x1: 'Raster',
                       0x2: 'Vector',
                       0x3: 'Truetype',
                       }
        if choices != None:
            subtype = obj.Object('Enumeration', 0x28, vm = self.obj_vm, parent = self, choices = choices)
            ftype += " (" + str(subtype) + ")"

        return ftype

    def flags(self):
        """Returns the file's flags"""
        data = struct.pack('=I', self.FileFlags & self.FileFlagsMask)
        addr_space = addrspace.BufferAddressSpace(self.obj_vm.get_config(), 0, data)
        bitmap = {'Debug': 0,
                  'Prerelease': 1,
                  'Patched': 2,
                  'Private Build': 3,
                  'Info Inferred': 4,
                  'Special Build' : 5,
                 }
        return obj.Object('Flags', offset = 0, vm = addr_space, bitmap = bitmap)

    def v(self):
        """Returns the value of the structure"""
        val = ("File version    : {0}\n" +
               "Product version : {1}\n" +
               "Flags           : {2}\n" +
               "OS              : {3}\n" +
               "File Type       : {4}\n" +
               "File Date       : {5}").format(self.file_version(), self.product_version(),
                                                 self.flags(), self.FileOS, self.file_type(), self.FileDate or '')
        return val

class _IMAGE_RESOURCE_DIR_STRING_U(obj.CType):
    """Handles Unicode-esque strings in IMAGE_RESOURCE_DIRECTORY structures"""
    # This is very similar to a UNICODE object, perhaps they should be merged somehow?
    def v(self):
        """Value function for _IMAGE_RESOURCE_DIR_STRING_U"""
        try:
            length = self.Length.v()
            if length > 1024:
                length = 0
            data = self.obj_vm.read(self.Value.obj_offset, length)
            return data.decode("utf16", "ignore").encode("ascii", 'backslashreplace')
        except Exception, _e:
            return ''

class _IMAGE_RESOURCE_DIRECTORY(obj.CType):
    """Handles Directory Entries"""
    def __init__(self, theType = None, offset = None, vm = None, parent = None, *args, **kwargs):
        self.sectoffset = offset
        obj.CType.__init__(self, theType = theType, offset = offset, vm = vm, parent = parent, *args, **kwargs)

    def get_entries(self):
        """Gets a tree of the entries from the top level IRD"""
        for irde in self.Entries:
            if irde != None:
                if irde.Name & 0x80000000:
                    # Points to a Name object
                    name = obj.Object("_IMAGE_RESOURCE_DIR_STRING_U", (irde.Name & 0x7FFFFFFF) + self.sectoffset, vm = self.obj_vm, parent = irde)
                else:
                    name = int(irde.Name)
                if irde.DataOffset & 0x80000000:
                    # We're another DIRECTORY
                    retobj = obj.Object("_IMAGE_RESOURCE_DIRECTORY", (irde.DataOffset & 0x7FFFFFFF) + self.sectoffset, vm = self.obj_vm, parent = irde)
                    retobj.sectoffset = self.sectoffset
                else:
                    # We're a DATA_ENTRY
                    retobj = obj.Object("_IMAGE_RESOURCE_DATA_ENTRY", irde.DataOffset + self.sectoffset, vm = self.obj_vm, parent = irde)
                yield (name, bool(irde.DataOffset & 0x80000000), retobj)

resource_types = {
 'RT_CURSOR'       : 1,
 'RT_BITMAP'       : 2,
 'RT_ICON'         : 3,
 'RT_MENU'         : 4,
 'RT_DIALOG'       : 5,
 'RT_STRING'       : 6,
 'RT_FONTDIR'      : 7,
 'RT_FONT'         : 8,
 'RT_ACCELERATOR'  : 9,
 'RT_RCDATA'       : 10,
 'RT_MESSAGETABLE' : 11,
 'RT_GROUP_CURSOR' : 12,
 'RT_GROUP_ICON'   : 14,
 'RT_VERSION'      : 16,
 'RT_DLGINCLUDE'   : 17,
 'RT_PLUGPLAY'     : 19,
 'RT_VXD'          : 20,
 'RT_ANICURSOR'    : 21,
 'RT_ANIICON'      : 22,
 'RT_HTML'         : 23,
}

class VerInfo(procdump.ProcExeDump):
    """Prints out the version information from PE images"""

    def __init__(self, config, *args, **kwargs):
        procdump.ProcExeDump.__init__(self, config, *args, **kwargs)
        config.remove_option("OFFSET")
        config.remove_option("PID")
        config.add_option("OFFSET", short_option = "o", type = 'int',
                          help = "Offset of the module to print the version information for")
        config.add_option('REGEX', short_option = "r", default = None,
                          help = 'Dump modules matching REGEX')
        config.add_option('IGNORE-CASE', short_option = 'i', action = 'store_true',
                      help = 'ignore case in pattern match', default = False)

    def calculate(self):
        """Returns a unique list of modules"""
        addr_space = utils.load_as(self._config)
        for cls in [_IMAGE_RESOURCE_DIRECTORY, _IMAGE_RESOURCE_DIR_STRING_U, _VS_FIXEDFILEINFO, _VS_VERSION_INFO, VerStruct]:
            addr_space.profile.object_classes[cls.__name__] = cls
        addr_space.profile.add_types(ver_types)

        if self._config.REGEX is not None:
            try:
                if self._config.IGNORE_CASE:
                    module_pattern = re.compile(self._config.REGEX, flags = sre_constants.SRE_FLAG_IGNORECASE)
                else:
                    module_pattern = re.compile(self._config.REGEX)
            except sre_constants.error, e:
                debug.error('Regular expression parsing error: {0}'.format(e))

        if self._config.OFFSET is not None:
            if not addr_space.is_valid_address(self._config.OFFSET):
                debug.error("Specified offset is not valid for the provided address space")
            yield addr_space, self._config.OFFSET
            raise StopIteration

        tasks = win32.tasks.pslist(addr_space)

        for task in tasks:
            for m in task.get_load_modules():
                if self._config.REGEX is not None:
                    if not (module_pattern.search(str(m.FullDllName))
                            or module_pattern.search(str(m.BaseDllName))):
                        continue

                yield task.get_process_address_space(), m

    def display_unicode(self, string):
        """Renders a UTF16 string"""
        if string is None:
            return ''
        return string.decode("utf16", "ignore").encode("ascii", 'backslashreplace')

    def get_version_info(self, addr_space, offset):
        """Accepts an address space and an executable image offset
        
           Returns a VS_VERSION_INFO object of NoneObject
        """
        if not addr_space.is_valid_address(offset):
            return obj.NoneObject("Disk image not resident in memory")

        try:
            nt_header = self.get_nt_header(addr_space = addr_space,
                                       base_addr = offset)
        except ValueError, ve:
            return obj.NoneObject("PE file failed initial sanity checks: {0}".format(ve))
        except exceptions.SanityCheckException, ve:
            return obj.NoneObject("PE file failed initial sanity checks: {0}. Try -u or --unsafe".format(ve))

        # header = s.read(m.DllBase, nt_header.OptionalHeader.SizeOfHeaders)

        for sect in nt_header.get_sections(self._config.UNSAFE):
            if str(sect.Name) == '.rsrc':
                root = obj.Object("_IMAGE_RESOURCE_DIRECTORY", offset + sect.VirtualAddress, addr_space)
                for rname, rentry, rdata in root.get_entries():
                    # We're a VERSION resource and we have subelements
                    if rname == resource_types['RT_VERSION'] and rentry:
                        for sname, sentry, sdata in rdata.get_entries():
                            # We're the single sub element of the VERSION
                            if sname == 1 and sentry:
                                # Get the string tables
                                for _stname, stentry, stdata in sdata.get_entries():
                                    if not stentry:
                                        return obj.Object("_VS_VERSION_INFO", offset = (stdata.DataOffset + offset), vm = addr_space)

    def render_text(self, outfd, data):
        """Renders the text"""
        for s, m in data:
            outfd.write(str(m.FullDllName))
            outfd.write("\n")
            vinfo = self.get_version_info(s, m.DllBase)
            if vinfo != None:
                outfd.write("  File version    : {0}\n".format(vinfo.FileInfo.file_version()))
                outfd.write("  Product version : {0}\n".format(vinfo.FileInfo.product_version()))
                outfd.write("  Flags           : {0}\n".format(vinfo.FileInfo.flags()))
                outfd.write("  OS              : {0}\n".format(vinfo.FileInfo.FileOS))
                outfd.write("  File Type       : {0}\n".format(vinfo.FileInfo.file_type()))
                outfd.write("  File Date       : {0}\n".format(vinfo.FileInfo.FileDate or ''))
                for name, children in vinfo.get_children():
                    if name == 'StringFileInfo':
                        for _codepage, strings in children:
                            for string, value in strings:
                                # Make sure value isn't a generator, and we've a subtree to deal with
                                if isinstance(value, type(strings)):
                                    outfd.write("  {0} : Subtrees not yet implemented\n".format(string))
                                else:
                                    outfd.write("  {0} : {1}\n".format(string, self.display_unicode(value)))



# Volatility
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License Version 2 as
# published by the Free Software Foundation.  You may not use, modify or
# distribute this program under any other version of the GNU General
# Public License.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

"""
@author:       Joe Sylve
@license:      GNU General Public License 2.0
@contact:      joe.sylve@gmail.com
@organization: Digital Forensics Solutions
"""

import volatility.plugins.linux.common as linux_common
import volatility.plugins.linux.mount as linux_mount
import volatility.plugins.linux.pslist as linux_pslist
from volatility.plugins.linux.slab_info import linux_slabinfo

class linux_mount_cache(linux_mount.linux_mount):
    """Gather mounted fs/devices from kmem_cache"""

    def __init__(self, config, *args, **kwargs):
        linux_mount.linux_mount.__init__(self, config, *args, **kwargs)
        self._config.add_option('UNALLOCATED', short_option = 'u',
                        default = False,
                        help = 'Show unallocated',
                        action = 'store_true')

    def calculate(self):
        linux_common.set_plugin_members(self)
    
        # newer kernels
        if self.profile.has_type("mount"):
            mnttype = "mount"
        
            cache = linux_slabinfo(self._config).get_kmem_cache(mnttype, self._config.UNALLOCATED)

            for task in linux_pslist.linux_pslist(self._config).calculate():
                if task.pid == 1:
                    ns = task.nsproxy.mnt_ns
                    break
        else:
            cache = linux_slabinfo(self._config).get_kmem_cache("mnt_cache", self._config.UNALLOCATED, struct_name = "vfsmount")
            ns = None

        for mnt in cache:
            yield (mnt, ns)




# Volatility
# Copyright (C) 2007-2013 Volatility Foundation
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License Version 2 as
# published by the Free Software Foundation.  You may not use, modify or
# distribute this program under any other version of the GNU General
# Public License.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

import volatility.plugins.crashinfo as crashinfo
import volatility.debug as debug

class HPAKInfo(crashinfo.CrashInfo):
    """Info on an HPAK file"""
    
    target_as = ['HPAKAddressSpace'] 
    
    def render_text(self, outfd, data):
        
        header = data.get_header()
        
        for section in header.Sections():
            outfd.write("Header:     {0}\n".format(section.Header))
            outfd.write("Length:     {0:#x}\n".format(section.Length))
            outfd.write("Offset:     {0:#x}\n".format(section.Offset))
            outfd.write("NextOffset: {0:#x}\n".format(section.NextSection))
            outfd.write("Name:       {0}\n".format(section.Name))
            outfd.write("Compressed: {0}\n".format(section.Compressed))
            outfd.write("\n")
            
class HPAKExtract(HPAKInfo):
    """Extract physical memory from an HPAK file"""
    
    def render_text(self, outfd, data):
            
        if not self._config.OUTPUT_FILE:
            debug.error("You must supply --output-file")
            
        header = data.get_header()

        data.convert_to_raw(outfd)
        
        print "Done."



# Volatility
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License Version 2 as
# published by the Free Software Foundation.  You may not use, modify or
# distribute this program under any other version of the GNU General
# Public License.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

"""
@author:       Joe Sylve
@license:      GNU General Public License 2.0
@contact:      joe.sylve@gmail.com
@organization: 504ENSICS Labs
"""

import volatility.obj as obj
import volatility.debug as debug
import volatility.plugins.linux.common as linux_common

class linux_check_evt_arm(linux_common.AbstractLinuxARMCommand):
    ''' Checks the Exception Vector Table to look for syscall table hooking '''

    VECTOR_BASE = 0xffff0000
    SWI_BASE = VECTOR_BASE + 8
        
    def calculate(self):
        linux_common.set_plugin_members(self)
        # Get instructions executed when an inturrupt exception occurs
        swi = obj.Object("unsigned int", offset = self.SWI_BASE, vm = self.addr_space)
        
        # Get offset of address to vector_swi
        offset = (swi & 0x0fff) + 8
        
        # Verify that instruction hasn't been modified (should be: ldr pc, [pc, #???] (e59ff???))
        if (swi & 0xfffff000) == 0xe59ff000:
            yield ("SWI Offset Instruction", "PASS", "Offset: {0}".format(offset))
        else:
            yield ("SWI Offset Instruction", "FAIL", "{0:X}".format(swi))
            return
        
        # Get vector_swi_addr from table
        vector_swi_addr = obj.Object("unsigned int", offset = self.SWI_BASE + (offset), vm = self.addr_space)
        
        # Check to see if vector_swi handler has been hooked
        if vector_swi_addr == self.addr_space.profile.get_symbol("vector_swi"):
            yield ("vector_swi address", "PASS", "0x{0:X}".format(vector_swi_addr))
        else:
            yield ("vector_swi address", "FAIL", "0x{0:X}".format(vector_swi_addr))
            return
            
        # Check for hooking of sys_call table pointer
        sc_opcode = None;
        max_opcodes_to_check = 1024
        while (max_opcodes_to_check):
            opcode = obj.Object("unsigned int", offset=  vector_swi_addr, vm = self.addr_space)
            if ((opcode & 0xffffff00) == 0xe28f8000):
                sc_opcode = opcode
                break
            vector_swi_addr += 4
            max_opcodes_to_check -= 1
            
        if sc_opcode:
            yield ("vector_swi code modification", "PASS", "{0:X}".format(sc_opcode))
        else:
            yield ("vector_swi code modification", "FAIL", "Opcode E28F80?? not found")
            return
          
    def render_text(self, outfd, data):
        self.table_header(outfd, [("Check", "<30"), ("PASS/FAIL", "<5"), ("Info", "<30")])
        for (check, result, info) in data:
            self.table_row(outfd, check, result, info)



# Volatility
# Copyright (C) 2007-2013 Volatility Foundation
# Copyright (C) 2010,2011,2012 Michael Hale Ligh <michael.ligh@mnin.org>
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License Version 2 as
# published by the Free Software Foundation.  You may not use, modify or
# distribute this program under any other version of the GNU General
# Public License.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

import volatility.obj as obj
import volatility.utils as utils
import volatility.plugins.common as common
import volatility.win32.modules as modules
import volatility.win32.tasks as tasks

class SessionsMixin(object):
    """This is a mixin that plugins can inherit for access to the 
    main sessions APIs."""

    def session_spaces(self, kernel_space):
        """ Generators unique _MM_SESSION_SPACE objects
        referenced by active processes. 
    
        @param space: a kernel AS for process enumeration
    
        @yields _MM_SESSION_SPACE instantiated from the 
        session space native_vm. 
        """
        seen = []
        for proc in tasks.pslist(kernel_space):
            if proc.SessionId != None and proc.SessionId.v() not in seen:
                ps_ad = proc.get_process_address_space()
                if ps_ad != None:
                    seen.append(proc.SessionId.v())
                    yield obj.Object("_MM_SESSION_SPACE",
                        offset = proc.Session.v(), vm = ps_ad)

    def find_session_space(self, kernel_space, session_id):
        """ Get a session address space by its ID. 
    
        @param space: a kernel AS for process enumeration
        @param session_id: the session ID to find.
    
        @returns _MM_SESSION_SPACE instantiated from the 
        session space native_vm. 
        """
        for proc in tasks.pslist(kernel_space):
            if proc.SessionId == session_id:
                ps_ad = proc.get_process_address_space()
                if ps_ad != None:
                    return obj.Object("_MM_SESSION_SPACE",
                        offset = proc.Session.v(), vm = ps_ad)
        return obj.NoneObject("Cannot locate a session")

class Sessions(common.AbstractWindowsCommand, SessionsMixin):
    """List details on _MM_SESSION_SPACE (user logon sessions)"""

    def calculate(self):
        kernel_space = utils.load_as(self._config)

        # Once for each unique _MM_SESSION_SPACE 
        for session in self.session_spaces(kernel_space):
            yield session

    def render_text(self, outfd, data):

        # Kernel AS for looking up modules 
        kernel_space = utils.load_as(self._config)

        # Modules sorted for address lookups 
        mods = dict((kernel_space.address_mask(mod.DllBase), mod) for mod in modules.lsmod(kernel_space))
        mod_addrs = sorted(mods.keys())

        for session in data:
            outfd.write("*" * 50 + "\n")
            outfd.write("Session(V): {0:x} ID: {1} Processes: {2}\n".format(
                session.obj_offset,
                session.SessionId,
                len(list(session.processes())),
                ))
            outfd.write("PagedPoolStart: {0:x} PagedPoolEnd {1:x}\n".format(
                session.PagedPoolStart,
                session.PagedPoolEnd,
                ))
            for process in session.processes():
                outfd.write(" Process: {0} {1} {2}\n".format(
                    process.UniqueProcessId,
                    process.ImageFileName,
                    process.CreateTime,
                    ))
            for image in session.images():
                module = tasks.find_module(mods, mod_addrs, kernel_space.address_mask(image.Address))
                outfd.write(" Image: {0:#x}, Address {1:x}, Name: {2}\n".format(
                    image.obj_offset,
                    image.Address,
                    str(module and module.BaseDllName or '')
                    ))



# Volatility
# Copyright (C) 2007-2013 Volatility Foundation 
# Copyright (C) 2010,2011,2012 Michael Hale Ligh <michael.ligh@mnin.org>
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License Version 2 as
# published by the Free Software Foundation.  You may not use, modify or
# distribute this program under any other version of the GNU General
# Public License.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

import volatility.plugins.gui.sessions as sessions
import volatility.debug as debug

class UserHandles(sessions.Sessions):
    """Dump the USER handle tables"""

    def __init__(self, config, *args, **kwargs):

        sessions.Sessions.__init__(self, config, *args, **kwargs)

        config.add_option('PID', short_option = 'p',
                help = 'Pid filter', action = 'store',
                type = 'int')

        config.add_option('TYPE', short_option = 't',
                help = 'Handle type', action = 'store',
                type = 'string')

        config.add_option('FREE', short_option = 'F',
                help = 'Include free handles', action = 'store_true',
                default = False)

    def render_text(self, outfd, data):

        for session in data:
            shared_info = session.find_shared_info()

            if not shared_info:
                debug.debug("Cannot find win32k!gSharedInfo")
                continue

            outfd.write("*" * 50 + "\n")
            outfd.write("SharedInfo: {0:#x}, SessionId: {1} Shared delta: {2}\n".format(
                shared_info.obj_offset, session.SessionId,
                shared_info.ulSharedDelta,
            ))
            outfd.write("aheList: {0:#x}, Table size: {1:#x}, Entry size: {2:#x}\n".format(
                shared_info.aheList.v(),
                shared_info.psi.cbHandleTable,
                shared_info.HeEntrySize if hasattr(shared_info, 'HeEntrySize') else shared_info.obj_vm.profile.get_obj_size("_HANDLEENTRY"),
            ))
            outfd.write("\n")

            filters = []

            # Should we display freed handles
            if not self._config.FREE:
                filters.append(lambda x : not x.Free)

            # Should we filter by process ID
            if self._config.PID:
                filters.append(lambda x : x.Process.UniqueProcessId == self._config.PID)

            # Should we filter by object type
            if self._config.TYPE:
                filters.append(lambda x : str(x.bType) == self._config.TYPE)

            self.table_header(outfd,
                         [("Object(V)", "[addrpad]"),
                          ("Handle", "[addr]"),
                          ("bType", "20"),
                          ("Flags", "^8"),
                          ("Thread", "^8"),
                          ("Process", ""),
                         ])

            for handle in shared_info.handles(filters):

                self.table_row(outfd,
                               handle.phead.v(),
                               handle.phead.h if handle.phead else 0,
                               handle.bType,
                               handle.bFlags,
                               handle.Thread.Cid.UniqueThread,
                               handle.Process.UniqueProcessId)



# Volatility
#
# Authors
# Michael Cohen <scudette@users.sourceforge.net>
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License Version 2 as
# published by the Free Software Foundation.  You may not use, modify or
# distribute this program under any other version of the GNU General
# Public License.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

"""pstree example file"""

import volatility.win32.tasks as tasks
import volatility.utils as utils
import volatility.plugins.common as common
import volatility.cache as cache
import volatility.obj as obj

#pylint: disable-msg=C0111

class ProcessAuditVTypes(obj.ProfileModification):
    before = ["WindowsVTypes"]
    conditions = {'os': lambda x: x == 'windows'}
    def modification(self, profile):
        profile.vtypes.update({
            '_SE_AUDIT_PROCESS_CREATION_INFO' : [ 0x4, {
            'ImageFileName' : [ 0x0, ['pointer', ['_OBJECT_NAME_INFORMATION']]],
            }],
            '_OBJECT_NAME_INFORMATION' : [ 0x8, {
            'Name' : [ 0x0, ['_UNICODE_STRING']],
            }]})

class PSTree(common.AbstractWindowsCommand):
    """Print process list as a tree"""

    def find_root(self, pid_dict, pid):
        # Prevent circular loops.
        seen = set()

        while pid in pid_dict and pid not in seen:
            seen.add(pid)
            pid = int(pid_dict[pid].InheritedFromUniqueProcessId)

        return pid

    def render_text(self, outfd, data):

        self.table_header(outfd, 
                         [("Name", "<50"), 
                          ("Pid", ">6"),
                          ("PPid", ">6"),
                          ("Thds", ">6"),
                          ("Hnds", ">6"),
                          ("Time", "")])

        def draw_branch(pad, inherited_from):
            for task in data.values():
                if task.InheritedFromUniqueProcessId == inherited_from:

                    first_column = "{0} {1:#x}:{2:20}".format(
                                        "." * pad, 
                                        task.obj_offset, 
                                        str(task.ImageFileName or '')
                                        )

                    self.table_row(outfd, 
                        first_column,
                        task.UniqueProcessId,
                        task.InheritedFromUniqueProcessId,
                        task.ActiveThreads,
                        task.ObjectTable.HandleCount,
                        task.CreateTime)

                    if self._config.VERBOSE:
                        outfd.write("{0}    audit: {1}\n".format(
                                ' ' * pad, str(task.SeAuditProcessCreationInfo.ImageFileName.Name or '')))
                        process_params = task.Peb.ProcessParameters
                        if process_params:
                            outfd.write("{0}    cmd: {1}\n".format(
                                ' ' * pad, str(process_params.CommandLine or '')))
                            outfd.write("{0}    path: {1}\n".format(
                                ' ' * pad, str(process_params.ImagePathName or '')))

                    del data[int(task.UniqueProcessId)]
                    draw_branch(pad + 1, task.UniqueProcessId) 

        while len(data.keys()) > 0:
            keys = data.keys()
            root = self.find_root(data, keys[0])
            draw_branch(0, root)

    @cache.CacheDecorator(lambda self: "tests/pstree/verbose={0}".format(self._config.VERBOSE))
    def calculate(self):

        ## Load a new address space
        addr_space = utils.load_as(self._config)

        return dict(
                (int(task.UniqueProcessId), task) 
                for task in tasks.pslist(addr_space)
                )



# Volatility
# Copyright (C) 2007-2013 Volatility Foundation
# Copyright (C) 2010,2011,2012 Michael Hale Ligh <michael.ligh@mnin.org>
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License Version 2 as
# published by the Free Software Foundation.  You may not use, modify or
# distribute this program under any other version of the GNU General
# Public License.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

import volatility.obj as obj
import volatility.scan as scan
import volatility.utils as utils
import volatility.plugins.filescan as filescan
import volatility.plugins.common as common
import volatility.plugins.gui.sessions as sessions

class PoolScanWind(scan.PoolScanner):
    """PoolScanner for window station objects"""

    def object_offset(self, found, address_space):
        """ This returns the offset of the object contained within
        this pool allocation.
        """
        pool_base = found - \
                self.buffer.profile.get_obj_offset('_POOL_HEADER', 'PoolTag')

        pool_obj = obj.Object("_POOL_HEADER", vm = address_space,
                                 offset = pool_base)

        pool_alignment = obj.VolMagic(address_space).PoolAlignment.v()

        object_base = (pool_base + pool_obj.BlockSize * pool_alignment -
                       common.pool_align(address_space,
                      'tagWINDOWSTATION', pool_alignment))

        return object_base

    checks = [ ('PoolTagCheck', dict(tag = "Win\xe4")),
               # seen as 0x98 on xpsp2 and xpsp3, 0x90 on w2k3*, 0xa0 on w7sp0
               ('CheckPoolSize', dict(condition = lambda x: x >= 0x90)),
               # only look in non-paged or free pools
               ('CheckPoolType', dict(paged = False, non_paged = True, free = True)),
               ('CheckPoolIndex', dict(value = 0)),
               ]

class WndScan(filescan.FileScan, sessions.SessionsMixin):
    """Pool scanner for tagWINDOWSTATION (window stations)"""

    def calculate(self):
        flat_space = utils.load_as(self._config, astype = 'physical')
        kernel_space = utils.load_as(self._config)

        # Scan for window station objects 
        for offset in PoolScanWind().scan(flat_space):

            window_station = obj.Object("tagWINDOWSTATION",
                offset = offset, vm = flat_space)

            # Basic sanity checks are included here 
            if not window_station.is_valid():
                continue

            # Find an address space for this window station's session  
            session = self.find_session_space(
                kernel_space, window_station.dwSessionId)

            if not session:
                continue

            # Reset the object's native VM so pointers are
            # dereferenced in session space 
            window_station.set_native_vm(session.obj_vm)

            for winsta in window_station.traverse():
                if winsta.is_valid():
                    yield winsta

    def render_text(self, outfd, data):

        seen = []

        for window_station in data:

            offset = window_station.PhysicalAddress
            if offset in seen:
                continue
            seen.append(offset)

            outfd.write("*" * 50 + "\n")
            outfd.write("WindowStation: {0:#x}, Name: {1}, Next: {2:#x}\n".format(
                offset,
                window_station.Name,
                window_station.rpwinstaNext.v(),
                ))
            outfd.write("SessionId: {0}, AtomTable: {1:#x}, Interactive: {2}\n".format(
                window_station.dwSessionId,
                window_station.pGlobalAtomTable,
                window_station.Interactive,
                ))
            outfd.write("Desktops: {0}\n".format(
                ', '.join([desk.Name for desk in window_station.desktops()])
                ))
            outfd.write("ptiDrawingClipboard: pid {0} tid {1}\n".format(
                window_station.ptiDrawingClipboard.pEThread.Cid.UniqueProcess,
                window_station.ptiDrawingClipboard.pEThread.Cid.UniqueThread
                ))
            outfd.write("spwndClipOpen: {0:#x}, spwndClipViewer: {1:#x} {2} {3}\n".format(
                window_station.spwndClipOpen.v(),
                window_station.spwndClipViewer.v(),
                str(window_station.LastRegisteredViewer.UniqueProcessId or ""),
                str(window_station.LastRegisteredViewer.ImageFileName or ""),
                ))
            outfd.write("cNumClipFormats: {0}, iClipSerialNumber: {1}\n".format(
                window_station.cNumClipFormats,
                window_station.iClipSerialNumber,
                ))
            outfd.write("pClipBase: {0:#x}, Formats: {1}\n".format(
                window_station.pClipBase,
                ",".join([str(clip.fmt) for clip in window_station.pClipBase.dereference()]),
                ))



# Volatility
# Copyright (C) 2008-2013 Volatility Foundation
# Copyright (c) 2008 Brendan Dolan-Gavitt <bdolangavitt@wesleyan.edu>
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License Version 2 as
# published by the Free Software Foundation.  You may not use, modify or
# distribute this program under any other version of the GNU General
# Public License.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

"""
@author:       AAron Walters and Brendan Dolan-Gavitt
@license:      GNU General Public License 2.0
@contact:      awalters@4tphi.net,bdolangavitt@wesleyan.edu
@organization: Volatility Foundation
"""

#pylint: disable-msg=C0111

import volatility.scan as scan
import volatility.obj as obj
import volatility.utils as utils
import volatility.plugins.common as common
import volatility.cache as cache

class CheckHiveSig(scan.ScannerCheck):
    """ Check for a registry hive signature """
    def check(self, offset):

        # Instead of hard-coding 4 here, calculate it safely in case 
        # additional fields are added to _POOL_HEADER after the pool tag. 
        offset += (self.address_space.profile.get_obj_size("_POOL_HEADER") -
                   self.address_space.profile.get_obj_offset("_POOL_HEADER", "PoolTag"))

        # We don't need to use pool alignment here because we're not 
        # carving from the bottom-up like other objects. There is no
        # object header or optional headers for _HHIVE. 

        sig = obj.Object('_HHIVE', vm = self.address_space, offset = offset).Signature
        return sig == 0xbee0bee0

class PoolScanHiveFast2(scan.PoolScanner):

    def object_offset(self, found, address_space):
        return found + (address_space.profile.get_obj_size("_POOL_HEADER") -
                        address_space.profile.get_obj_offset("_POOL_HEADER", "PoolTag"))

    checks = [ ('PoolTagCheck', dict(tag = "CM10")),
               # Dummy condition, since this will be changed during initialization
               ('CheckPoolSize', dict(condition = lambda x: x == 0x638)),
               #('CheckPoolType', dict(non_paged = True)), #doesn't work for win7 and vista
               ('CheckHiveSig', {})
               ]

    def __init__(self, poolsize):
        self.checks[1] = ('CheckPoolSize', dict(condition = lambda x: x >= poolsize))
        scan.PoolScanner.__init__(self)

class HiveScan(common.AbstractWindowsCommand):
    """ Scan Physical memory for _CMHIVE objects (registry hives)

    You will need to obtain these offsets to feed into the hivelist command.
    """

    # Declare meta information associated with this plugin

    meta_info = dict(
        author = 'Brendan Dolan-Gavitt',
        copyright = 'Copyright (c) 2007,2008 Brendan Dolan-Gavitt',
        contact = 'bdolangavitt@wesleyan.edu',
        license = 'GNU General Public License 2.0',
        url = 'http://moyix.blogspot.com/',
        os = 'WIN_32_XP_SP2',
        version = '1.0',
        )

    @cache.CacheDecorator("tests/hivescan")
    def calculate(self):
        ## Just grab the AS and scan it using our scanner
        pspace = utils.load_as(self._config, astype = 'physical')
        poolsize = pspace.profile.get_obj_size('_CMHIVE')

        return PoolScanHiveFast2(poolsize).scan(pspace)

    def render_text(self, outfd, data):
        self.table_header(outfd, [('Offset(P)', '[addrpad]')])
        for offset in data:
            self.table_row(outfd, offset)



# Volatility
# Copyright (C) 2007-2013 Volatility Foundation
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License Version 2 as
# published by the Free Software Foundation.  You may not use, modify or
# distribute this program under any other version of the GNU General
# Public License.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

"""
@author:       Andrew Case
@license:      GNU General Public License 2.0
@contact:      atcuno@gmail.com
@organization:
"""

import volatility.obj as obj
import volatility.plugins.linux.common as linux_common

class linux_iomem(linux_common.AbstractLinuxCommand):
    """Provides output similar to /proc/iomem"""

    def yield_resource(self, io_ptr, depth = 0):

        if not io_ptr:
            #print "null"
            return []

        io_res = obj.Object("resource", offset = io_ptr, vm = self.addr_space)

        name = io_res.name.dereference_as("String", length = linux_common.MAX_STRING_LENGTH)
        start = io_res.start
        end = io_res.end

        output = [(depth, name, start, end)]

        output += self.yield_resource(io_res.child, depth + 1)
        output += self.yield_resource(io_res.sibling, depth)
        return output

    def calculate(self):
        linux_common.set_plugin_members(self)

        io_ptr = self.addr_space.profile.get_symbol("iomem_resource")

        for r in self.yield_resource(io_ptr):
            yield r

    def render_text(self, outfd, data):

        for output in data:
            depth, name, start, end = output
            outfd.write("{0:35s}\t0x{1:<16X}\t0x{2:<16X}\n".format(("  " * depth) + name, start, end))



# Volatility
# Copyright (C) 2007-2013 Volatility Foundation
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License Version 2 as
# published by the Free Software Foundation.  You may not use, modify or
# distribute this program under any other version of the GNU General
# Public License.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

"""
@author:       Andrew Case
@license:      GNU General Public License 2.0
@contact:      atcuno@gmail.com
@organization: 
"""

import os
import volatility.obj as obj
import volatility.plugins.linux.common as linux_common
import volatility.plugins.linux.lsof as linux_lsof
import volatility.plugins.linux.lsmod as linux_lsmod

class linux_check_afinfo(linux_common.AbstractLinuxCommand):
    """Verifies the operation function pointers of network protocols"""

    def check_members(self, var_ops, var_name, members, modules):

        for (hooked_member, hook_address) in self.verify_ops(var_ops, members, modules):
            yield (hooked_member, hook_address)

    def check_afinfo(self, var_name, var, op_members, seq_members, modules):

        for (hooked_member, hook_address) in self.check_members(var.seq_fops, var_name, op_members,  modules):
            yield (var_name, hooked_member, hook_address)

        # newer kernels
        if hasattr(var, "seq_ops"):
            for (hooked_member, hook_address) in self.check_members(var.seq_ops, var_name, seq_members, modules):
                yield (var_name, hooked_member, hook_address) 
                
        elif not self.is_known_address(var.seq_show, modules):
            yield (var_name, "show", var.seq_show)

    def calculate(self):
        linux_common.set_plugin_members(self)
        
        modules  = linux_lsmod.linux_lsmod(self._config).get_modules()
        op_members  = self.profile.types['file_operations'].keywords["members"].keys()
        seq_members = self.profile.types['seq_operations'].keywords["members"].keys()       

        tcp = ("tcp_seq_afinfo", ["tcp6_seq_afinfo", "tcp4_seq_afinfo"])
        udp = ("udp_seq_afinfo", ["udplite6_seq_afinfo", "udp6_seq_afinfo", "udplite4_seq_afinfo", "udp4_seq_afinfo"])
        protocols = [tcp, udp]

        for proto in protocols:
            
            struct_type = proto[0]

            for global_var_name in proto[1]:
                
                global_var_addr = self.addr_space.profile.get_symbol(global_var_name)

                if not global_var_addr:
                    continue

                global_var = obj.Object(struct_type, offset = global_var_addr, vm = self.addr_space)

                for (name, member, address) in self.check_afinfo(global_var_name, global_var, op_members, seq_members, modules):
                    yield (name, member, address)
        
    def render_text(self, outfd, data):

        self.table_header(outfd, [("Symbol Name", "42"), 
                                  ("Member", "30"), 
                                  ("Address", "[addrpad]")])
                                  
        for (what, member, address) in data:
            self.table_row(outfd, what, member, address)





# Volatility
# Copyright (C) 2007-2013 Volatility Foundation
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License Version 2 as
# published by the Free Software Foundation.  You may not use, modify or
# distribute this program under any other version of the GNU General
# Public License.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

"""
@author:       Andrew Case
@license:      GNU General Public License 2.0
@contact:      atcuno@gmail.com
@organization: 
"""

import volatility.obj as obj
import volatility.plugins.linux.common as linux_common

class linux_dmesg(linux_common.AbstractLinuxCommand):
    """Gather dmesg buffer"""

    def _get_log_info(self):

        ptr_addr = self.addr_space.profile.get_symbol("log_buf")
        log_buf_addr = obj.Object("unsigned long", offset = ptr_addr, vm = self.addr_space)
        log_buf_len = obj.Object("int", self.addr_space.profile.get_symbol("log_buf_len"), vm = self.addr_space)

        return (log_buf_addr, log_buf_len)

    # pre 3.x
    def _pre_3(self, buf_addr, buf_len):

        return obj.Object("String", offset = buf_addr, vm = self.addr_space, length = buf_len)

    def _ver_3(self, buf_addr, buf_len):
        '''
        During 3.x, the kernel switched the kernel debug buffer from just a big char array to the variable now
        holding variable sized records tracked by inline 'log' structures
        We deal with this by walking all the logs and building the buffer up and then returning it
        This produces the same results as the old way
        '''

        ret = ""

        size_of_log = self.profile.get_obj_size("log")

        cur_addr = buf_addr
        end_addr = buf_addr + buf_len

        log = obj.Object("log", offset = cur_addr, vm = self.addr_space)
        cur_len = log.len

        while cur_addr < end_addr and cur_len != 0:

            msg_len = log.text_len
            cur_ts = log.ts_nsec

            buf = obj.Object("String", offset = cur_addr + size_of_log, vm = self.addr_space, length = msg_len)

            ret = ret + "[{0}.{1}] {2}\n".format(cur_ts, cur_ts / 1000000000, buf)

            cur_addr = cur_addr + cur_len

            log = obj.Object("log", offset = cur_addr, vm = self.addr_space)
            cur_len = log.len

        return ret

    def calculate(self):
        linux_common.set_plugin_members(self)
        (log_buf_addr, log_buf_len) = self._get_log_info()

        if self.profile.has_type("log") and self.profile.obj_has_member("log", "ts_nsec"):
            yield self._ver_3(log_buf_addr, log_buf_len)

        else:
            yield self._pre_3(log_buf_addr, log_buf_len)

    def render_text(self, outfd, data):

        for buf in data:
            outfd.write("{0:s}\n".format(buf))







# Volatility
# Copyright (C) 2007-2013 Volatility Foundation
# Copyright (C) 2010,2011,2012 Michael Hale Ligh <michael.ligh@mnin.org>
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License Version 2 as
# published by the Free Software Foundation.  You may not use, modify or
# distribute this program under any other version of the GNU General
# Public License.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

import volatility.obj as obj
import volatility.plugins.gui.constants as consts
import volatility.plugins.gui.win32k_core as win32k_core
import volatility.plugins.gui.vtypes.win7_sp0_x64_vtypes_gui as win7_sp0_x64_vtypes_gui
import volatility.plugins.gui.vtypes.win7_sp0_x86_vtypes_gui as win7_sp0_x86_vtypes_gui
import volatility.plugins.gui.vtypes.win7_sp1_x64_vtypes_gui as win7_sp1_x64_vtypes_gui
import volatility.plugins.gui.vtypes.win7_sp1_x86_vtypes_gui as win7_sp1_x86_vtypes_gui

class Win7SP0x64GuiVTypes(obj.ProfileModification):
    """Apply the base vtypes for Windows 7 SP0 x64"""

    conditions = {'os': lambda x: x == 'windows',
                  'memory_model': lambda x: x == '64bit',
                  'major': lambda x: x == 6,
                  'minor': lambda x: x == 1,
                  'build': lambda x : x == 7600}

    def modification(self, profile):
        profile.vtypes.update(win7_sp0_x64_vtypes_gui.win32k_types)

class Win7SP1x64GuiVTypes(obj.ProfileModification):
    """Apply the base vtypes for Windows 7 SP1 x64"""

    conditions = {'os': lambda x: x == 'windows',
                  'memory_model': lambda x: x == '64bit',
                  'major': lambda x: x == 6,
                  'minor': lambda x: x == 1,
                  'build': lambda x : x == 7601}

    def modification(self, profile):
        profile.vtypes.update(win7_sp1_x64_vtypes_gui.win32k_types)

class Win7SP0x86GuiVTypes(obj.ProfileModification):
    """Apply the base vtypes for Windows 7 SP0 x86"""

    conditions = {'os': lambda x: x == 'windows',
                  'memory_model': lambda x: x == '32bit',
                  'major': lambda x: x == 6,
                  'minor': lambda x: x == 1,
                  'build': lambda x : x == 7600}

    def modification(self, profile):
        profile.vtypes.update(win7_sp0_x86_vtypes_gui.win32k_types)

class Win7SP1x86GuiVTypes(obj.ProfileModification):
    """Apply the base vtypes for Windows 7 SP1 x86"""

    conditions = {'os': lambda x: x == 'windows',
                  'memory_model': lambda x: x == '32bit',
                  'major': lambda x: x == 6,
                  'minor': lambda x: x == 1,
                  'build': lambda x : x == 7601}

    def modification(self, profile):
        profile.vtypes.update(win7_sp1_x86_vtypes_gui.win32k_types)

class Win7GuiOverlay(obj.ProfileModification):
    """Apply general overlays for Windows 7"""

    before = ['Win7SP0x64GuiVTypes', 'Win7SP1x64GuiVTypes',
              'Win7SP0x86GuiVTypes', 'Win7SP1x86GuiVTypes']

    conditions = {'os': lambda x: x == 'windows',
                  'major': lambda x: x == 6,
                  'minor': lambda x: x == 1}

    def modification(self, profile):
        profile.merge_overlay({
            'tagHOOK': [ None, {
            'flags': [ None, ['Flags', {'bitmap': consts.HOOK_FLAGS}]]
            }],
            '_HANDLEENTRY': [ None, {
            'bType': [ None, ['Enumeration', dict(target = 'unsigned char', choices = consts.HANDLE_TYPE_ENUM_SEVEN)]],
            }],
            'tagWINDOWSTATION' : [ None, {
            'pClipBase' : [ None, ['pointer', ['array', lambda x : x.cNumClipFormats, ['tagCLIP']]]],
            }],
            'tagCLIP': [ None, {
            'fmt' : [ None, ['Enumeration', dict(target = 'unsigned long', choices = consts.CLIPBOARD_FORMAT_ENUM)]],
            }]})

class Win7Vista2008x64Timers(obj.ProfileModification):
    """Apply the tagTIMER for Windows 7, Vista, and 2008 x64"""

    conditions = {'os': lambda x: x == 'windows',
                  'memory_model': lambda x: x == '64bit',
                  'major': lambda x: x >= 6}

    def modification(self, profile):
        # http://doxygen.reactos.org/d5/dd0/timer_8h_source.html#l00019
        profile.vtypes.update({
            'tagTIMER' : [ None, {
            'head' : [ 0x00, ['_HEAD']],
            'ListEntry' : [ 0x18, ['_LIST_ENTRY']],
            'spwnd' : [ 0x28, ['pointer', ['tagWND']]],
            'pti' : [ 0x30, ['pointer', ['tagTHREADINFO']]],
            'nID' : [ 0x38, ['unsigned short']],
            'cmsCountdown' : [ 0x40, ['unsigned int']],
            'cmsRate' : [ 0x44, ['unsigned int']],
            'flags' : [ 0x48, ['Flags', {'bitmap': consts.TIMER_FLAGS}]],
            'pfn' : [ 0x50, ['pointer', ['void']]],
            }]})

class Win7Vista2008x86Timers(obj.ProfileModification):
    """Apply the tagTIMER for Windows 7, Vista, and 2008 x86"""

    conditions = {'os': lambda x: x == 'windows',
                  'memory_model': lambda x: x == '32bit',
                  'major': lambda x: x >= 6}

    def modification(self, profile):
        profile.vtypes.update({
            'tagTIMER' : [ None, {
            'ListEntry' : [ 0xc, ['_LIST_ENTRY']],
            'pti' : [ 0x18, ['pointer', ['tagTHREADINFO']]],
            'spwnd' : [ 0x14, ['pointer', ['tagWND']]], #??
            'nID' : [ 0x1C, ['unsigned short']],
            'cmsCountdown' : [ 0x20, ['unsigned int']],
            'cmsRate' : [ 0x24, ['unsigned int']],
            'flags' : [ 0x28, ['Flags', {'bitmap': consts.TIMER_FLAGS}]],
            'pfn' : [ 0x2C, ['pointer', ['void']]],
            }]})

class _MM_SESSION_SPACE(win32k_core._MM_SESSION_SPACE): #pylint: disable-msg=W0212
    """A class for session spaces on Windows 7"""

    def find_shared_info(self):
        """The way we find win32k!gSharedInfo on Windows 7
        is different than before. For each DWORD in the 
        win32k.sys module's .data section (DWORD-aligned)
        we check if its the HeEntrySize member of a possible
        tagSHAREDINFO structure. This should equal the size 
        of a _HANDLEENTRY.

        The HeEntrySize member didn't exist before Windows 7
        thus the need for separate methods."""

        handle_table_size = self.obj_vm.profile.\
                            get_obj_size("_HANDLEENTRY")

        handle_entry_offset = self.obj_vm.profile.\
                            get_obj_offset("tagSHAREDINFO", "HeEntrySize")

        for chunk in self._section_chunks(".data"):

            if chunk != handle_table_size:
                continue

            shared_info = obj.Object("tagSHAREDINFO",
                offset = chunk.obj_offset - handle_entry_offset,
                vm = self.obj_vm)

            if shared_info.is_valid():
                return shared_info

        return obj.NoneObject("Cannot find win32k!gSharedInfo")

class tagSHAREDINFO(win32k_core.tagSHAREDINFO):
    """A class for shared info blocks on Windows 7"""

    def is_valid(self):
        """Sanity checks for tagSHAREDINFO"""

        if not obj.CType.is_valid(self):
            return False

        if self.ulSharedDelta != 0:
            return False

        if not self.psi.is_valid():
            return False

        return self.psi.cbHandleTable / self.HeEntrySize == self.psi.cHandleEntries

class Win7Win32KCoreClasses(obj.ProfileModification):
    """Apply the core object classes for Windows 7"""

    before = ["WindowsObjectClasses", "Win32KCoreClasses"]

    conditions = {'os': lambda x: x == 'windows',
                  'major' : lambda x : x == 6,
                  'minor' : lambda x : x == 1}

    def modification(self, profile):

        profile.object_classes.update({
            '_MM_SESSION_SPACE': _MM_SESSION_SPACE,
            'tagSHAREDINFO': tagSHAREDINFO,
            })




# Volatility
#
# Authors:
# Michael Cohen <scudette@users.sourceforge.net>
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License Version 2 as
# published by the Free Software Foundation.  You may not use, modify or
# distribute this program under any other version of the GNU General
# Public License.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

""" General debugging framework """
import pdb
import sys
import inspect
import logging
import volatility.conf
config = volatility.conf.ConfObject()

config.add_option("DEBUG", short_option = 'd', default = 0,
                  cache_invalidator = False,
                  action = 'count', help = "Debug volatility")

# Largest debug value used + 1
MAX_DEBUG = 3

def setup(level = 0):
    """Sets up the global logging environment"""
    formatstr = "%(levelname)-8s: %(name)-20s: %(message)s"
    logging.basicConfig(format = formatstr)
    rootlogger = logging.getLogger('')
    rootlogger.setLevel(logging.DEBUG + 1 - level)
    for i in range(1, 9):
        logging.addLevelName(logging.DEBUG - i, "DEBUG" + str(i))

def debug(msg, level = 1):
    """Logs a message at the DEBUG level"""
    log(msg, logging.DEBUG + 1 - level)

def info(msg):
    """Logs a message at the INFO level"""
    log(msg, logging.INFO)

def warning(msg):
    """Logs a message at the WARNING level"""
    log(msg, logging.WARNING)

def error(msg):
    log(msg, logging.ERROR)
    sys.exit(1)

def critical(msg):
    log(msg, logging.CRITICAL)
    sys.exit(1)

def log(msg, level):
    modname = "volatility.py"
    try:
        frm = inspect.currentframe()
        modname = "volatility.debug"
        while modname == "volatility.debug":
            frm = frm.f_back
            mod = inspect.getmodule(frm)
            modname = mod.__name__
    except AttributeError:
        pass
    finally:
        del frm
    _log(msg, modname, level)

def _log(msg, facility, loglevel):
    """Outputs a debugging message"""
    logger = logging.getLogger(facility)
    logger.log(loglevel, msg)

def b(level = 1):
    """Enters the debugger at the call point"""
    if config.DEBUG >= level:
        pdb.set_trace()

trace = b

def post_mortem(level = 1):
    """Provides a command line interface to python after an exception's occurred"""
    if config.DEBUG >= level:
        pdb.post_mortem()



# Volatility
# Copyright (C) 2007-2013 Volatility Foundation
#
# Additional Authors:
# Michael Cohen <scudette@users.sourceforge.net>
# Mike Auty <mike.auty@gmail.com>
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License Version 2 as
# published by the Free Software Foundation.  You may not use, modify or
# distribute this program under any other version of the GNU General
# Public License.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

#pylint: disable-msg=C0111

import os
import volatility.plugins.common as common
import volatility.win32 as win32
import volatility.obj as obj
import volatility.debug as debug
import volatility.utils as utils
import volatility.cache as cache

class DllList(common.AbstractWindowsCommand, cache.Testable):
    """Print list of loaded dlls for each process"""

    def __init__(self, config, *args, **kwargs):
        common.AbstractWindowsCommand.__init__(self, config, *args, **kwargs)
        cache.Testable.__init__(self)
        config.add_option('OFFSET', short_option = 'o', default = None,
                          help = 'EPROCESS offset (in hex) in the physical address space',
                          action = 'store', type = 'int')

        config.add_option('PID', short_option = 'p', default = None,
                          help = 'Operate on these Process IDs (comma-separated)',
                          action = 'store', type = 'str')

    def render_text(self, outfd, data):
        for task in data:
            pid = task.UniqueProcessId

            outfd.write("*" * 72 + "\n")
            outfd.write("{0} pid: {1:6}\n".format(task.ImageFileName, pid))

            if task.Peb:
                outfd.write("Command line : {0}\n".format(str(task.Peb.ProcessParameters.CommandLine or '')))
                if task.IsWow64:
                    outfd.write("Note: use ldrmodules for listing DLLs in Wow64 processes\n")
                outfd.write("{0}\n".format(str(task.Peb.CSDVersion or '')))
                outfd.write("\n")
                self.table_header(outfd,
                                  [("Base", "[addrpad]"),
                                   ("Size", "[addr]"),
                                   ("LoadCount", "[addr]"), 
                                   ("Path", ""),
                                   ])
                for m in task.get_load_modules():
                    self.table_row(outfd, m.DllBase, m.SizeOfImage, m.LoadCount, str(m.FullDllName or ''))
            else:
                outfd.write("Unable to read PEB for task.\n")

    def filter_tasks(self, tasks):
        """ Reduce the tasks based on the user selectable PIDS parameter.

        Returns a reduced list or the full list if config.PIDS not specified.
        """
        
        if self._config.PID is None:
            return tasks
        
        try:
            pidlist = [int(p) for p in self._config.PID.split(',')]
        except ValueError:
            debug.error("Invalid PID {0}".format(self._config.PID))
            
        return [t for t in tasks if t.UniqueProcessId in pidlist]

    @staticmethod
    def virtual_process_from_physical_offset(addr_space, offset):
        """ Returns a virtual process from a physical offset in memory """
        # Since this is a physical offset, we find the process
        flat_addr_space = utils.load_as(addr_space.get_config(), astype = 'physical')
        flateproc = obj.Object("_EPROCESS", offset, flat_addr_space)
        # then use the virtual address of its first thread to get into virtual land
        # (Note: the addr_space and flat_addr_space use the same config, so should have the same profile)
        tleoffset = addr_space.profile.get_obj_offset("_ETHREAD", "ThreadListEntry")
        ethread = obj.Object("_ETHREAD", offset = flateproc.ThreadListHead.Flink.v() - tleoffset, vm = addr_space)
        # and ask for the thread's process to get an _EPROCESS with a virtual address space
        virtual_process = ethread.owning_process()
        # Sanity check the bounce. See Issue 154. 
        if virtual_process and offset == addr_space.vtop(virtual_process.obj_offset):
            return virtual_process
        return obj.NoneObject("Unable to bounce back from virtual _ETHREAD to virtual _EPROCESS")

    @cache.CacheDecorator(lambda self: "tests/pslist/pid={0}/offset={1}".format(self._config.PID, self._config.OFFSET))
    def calculate(self):
        """Produces a list of processes, or just a single process based on an OFFSET"""
        addr_space = utils.load_as(self._config)

        if self._config.OFFSET != None:
            tasks = [self.virtual_process_from_physical_offset(addr_space, self._config.OFFSET)]
        else:
            tasks = self.filter_tasks(win32.tasks.pslist(addr_space))

        return tasks

class PSList(DllList):
    """ Print all running processes by following the EPROCESS lists """
    def __init__(self, config, *args, **kwargs):
        DllList.__init__(self, config, *args, **kwargs)
        config.add_option("PHYSICAL-OFFSET", short_option = 'P', default = False,
                          cache_invalidator = False, help = "Physical Offset", action = "store_true")

    def render_text(self, outfd, data):

        offsettype = "(V)" if not self._config.PHYSICAL_OFFSET else "(P)"
        self.table_header(outfd,
                          [("Offset{0}".format(offsettype), "[addrpad]"),
                           ("Name", "20s"),
                           ("PID", ">6"),
                           ("PPID", ">6"),
                           ("Thds", ">6"),
                           ("Hnds", ">8"),
                           ("Sess", ">6"),
                           ("Wow64", ">6"),
                           ("Start", "30"),
                           ("Exit", "30")]
                          )

        for task in data:
            # PHYSICAL_OFFSET must STRICTLY only be used in the results.  If it's used for anything else,
            # it needs to have cache_invalidator set to True in the options
            if not self._config.PHYSICAL_OFFSET:
                offset = task.obj_offset
            else:
                offset = task.obj_vm.vtop(task.obj_offset)
            self.table_row(outfd,
                offset,
                task.ImageFileName,
                task.UniqueProcessId,
                task.InheritedFromUniqueProcessId,
                task.ActiveThreads,
                task.ObjectTable.HandleCount,
                task.SessionId,
                task.IsWow64,
                str(task.CreateTime or ''),
                str(task.ExitTime or ''),
                )


# Inherit from files just for the config options (__init__)
class MemMap(DllList):
    """Print the memory map"""

    def render_text(self, outfd, data):
        first = True
        for pid, task, pagedata in data:
            if not first:
                outfd.write("*" * 72 + "\n")

            task_space = task.get_process_address_space()
            outfd.write("{0} pid: {1:6}\n".format(task.ImageFileName, pid))
            first = False

            offset = 0
            if pagedata:
                self.table_header(outfd,
                                  [("Virtual", "[addrpad]"),
                                   ("Physical", "[addrpad]"),
                                   ("Size", "[addr]"),
                                   ("DumpFileOffset", "[addr]")])

                for p in pagedata:
                    pa = task_space.vtop(p[0])
                    # pa can be 0, according to the old memmap, but can't == None(NoneObject)
                    if pa != None:
                        self.table_row(outfd, p[0], pa, p[1], offset)
                    #else:
                    #    outfd.write("0x{0:10x} 0x000000     0x{1:12x}\n".format(p[0], p[1]))
                        offset += p[1]
            else:
                outfd.write("Unable to read pages for task.\n")

    @cache.CacheDecorator(lambda self: "tests/memmap/pid={0}/offset={1}".format(self._config.PID, self._config.OFFSET))
    def calculate(self):
        tasks = DllList.calculate(self)

        for task in tasks:
            if task.UniqueProcessId:
                pid = task.UniqueProcessId
                task_space = task.get_process_address_space()
                pages = task_space.get_available_pages()
                yield pid, task, pages

class MemDump(MemMap):
    """Dump the addressable memory for a process"""

    def __init__(self, config, *args, **kwargs):
        MemMap.__init__(self, config, *args, **kwargs)
        config.add_option('DUMP-DIR', short_option = 'D', default = None,
                          cache_invalidator = False,
                          help = 'Directory in which to dump memory')

    def render_text(self, outfd, data):
        if self._config.DUMP_DIR == None:
            debug.error("Please specify a dump directory (--dump-dir)")
        if not os.path.isdir(self._config.DUMP_DIR):
            debug.error(self._config.DUMP_DIR + " is not a directory")

        for pid, task, pagedata in data:
            outfd.write("*" * 72 + "\n")

            task_space = task.get_process_address_space()
            outfd.write("Writing {0} [{1:6}] to {2}.dmp\n".format(task.ImageFileName, pid, str(pid)))

            f = open(os.path.join(self._config.DUMP_DIR, str(pid) + ".dmp"), 'wb')
            if pagedata:
                for p in pagedata:
                    data = task_space.read(p[0], p[1])
                    if data == None:
                        if self._config.verbose:
                            outfd.write("Memory Not Accessible: Virtual Address: 0x{0:x} File Offset: 0x{1:x} Size: 0x{2:x}\n".format(p[0], task.obj_offset, p[1]))
                    else:
                        f.write(data)
            else:
                outfd.write("Unable to read pages for task.\n")
            f.close()



# Volatility
#
# Authors:
# Mike Auty <mike.auty@gmail.com>
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License Version 2 as
# published by the Free Software Foundation.  You may not use, modify or
# distribute this program under any other version of the GNU General
# Public License.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

#pylint: disable-msg=C0111

import volatility.plugins.common as common
import volatility.cache as cache
import volatility.win32 as win32
import volatility.utils as utils

class Modules(common.AbstractWindowsCommand):
    """Print list of loaded modules"""
    def __init__(self, config, *args, **kwargs):
        common.AbstractWindowsCommand.__init__(self, config, *args, **kwargs)
        config.add_option("PHYSICAL-OFFSET", short_option = 'P', default = False,
                          cache_invalidator = False, help = "Physical Offset", action = "store_true")

    def render_text(self, outfd, data):
        offsettype = "(V)" if not self._config.PHYSICAL_OFFSET else "(P)"
        self.table_header(outfd,
                          [("Offset{0}".format(offsettype), "[addrpad]"),
                           ("Name", "20"),
                           ('Base', "[addrpad]"),
                           ('Size', "[addr]"),
                           ('File', "")
                           ])

        for module in data:
            if not self._config.PHYSICAL_OFFSET:
                offset = module.obj_offset
            else:
                offset = module.obj_vm.vtop(module.obj_offset)
            self.table_row(outfd,
                         offset,
                         str(module.BaseDllName  or ''),
                         module.DllBase,
                         module.SizeOfImage,
                         str(module.FullDllName or ''))


    @cache.CacheDecorator("tests/lsmod")
    def calculate(self):
        addr_space = utils.load_as(self._config)

        result = win32.modules.lsmod(addr_space)

        return result

class UnloadedModules(common.AbstractWindowsCommand):
    """Print list of unloaded modules"""

    def render_text(self, outfd, data):

        self.table_header(outfd, [
                           ("Name", "20"),
                           ('StartAddress', "[addrpad]"),
                           ('EndAddress', "[addrpad]"),
                           ('Time', "")])

        for drv in data:
            self.table_row(outfd, drv.Name, drv.StartAddress, 
                          drv.EndAddress, drv.CurrentTime) 

    def calculate(self):
        addr_space = utils.load_as(self._config)
    
        kdbg = win32.tasks.get_kdbg(addr_space)

        for drv in kdbg.MmUnloadedDrivers.dereference().dereference():
            yield drv 



# Volatility
# Copyright (C) 2007-2013 Volatility Foundation
# 
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License Version 2 as
# published by the Free Software Foundation.  You may not use, modify or
# distribute this program under any other version of the GNU General
# Public License.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

"""
@author:       Andrew Case
@license:      GNU General Public License 2.0
@contact:      atcuno@gmail.com
@organization: 
"""

import volatility.obj as obj
import volatility.plugins.mac.common as common
import volatility.plugins.mac.route as route

class mac_arp(route.mac_route):
    """ Prints the arp table """
    
    def calculate(self):
        common.set_plugin_members(self)

        arp_addr = self.addr_space.profile.get_symbol("_llinfo_arp")
        ptr = obj.Object("Pointer", offset = arp_addr, vm = self.addr_space)
        ent = ptr.dereference_as("llinfo_arp") 

        while ent:
            yield ent.la_rt
            ent = ent.la_le.le_next



# Volatility
# Copyright (c) 2008-2013 Volatility Foundation
# Copyright (c) 2011 Michael Hale Ligh <michael.hale@gmail.com>
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License Version 2 as
# published by the Free Software Foundation.  You may not use, modify or
# distribute this program under any other version of the GNU General
# Public License.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

syscalls = [
    [
    'NtAcceptConnectPort', # 0x0
    'NtAccessCheck', # 0x1
    'NtAccessCheckAndAuditAlarm', # 0x2
    'NtAccessCheckByType', # 0x3
    'NtAccessCheckByTypeAndAuditAlarm', # 0x4
    'NtAccessCheckByTypeResultList', # 0x5
    'NtAccessCheckByTypeResultListAndAuditAlarm', # 0x6
    'NtAccessCheckByTypeResultListAndAuditAlarmByHandle', # 0x7
    'NtAddAtom', # 0x8
    'NtAddBootEntry', # 0x9
    'NtAddDriverEntry', # 0xa
    'NtAdjustGroupsToken', # 0xb
    'NtAdjustPrivilegesToken', # 0xc
    'NtAlertResumeThread', # 0xd
    'NtAlertThread', # 0xe
    'NtAllocateLocallyUniqueId', # 0xf
    'NtAllocateUserPhysicalPages', # 0x10
    'NtAllocateUuids', # 0x11
    'NtAllocateVirtualMemory', # 0x12
    'NtAlpcAcceptConnectPort', # 0x13
    'NtAlpcCancelMessage', # 0x14
    'NtAlpcConnectPort', # 0x15
    'NtAlpcCreatePort', # 0x16
    'NtAlpcCreatePortSection', # 0x17
    'NtAlpcCreateResourceReserve', # 0x18
    'NtAlpcCreateSectionView', # 0x19
    'NtAlpcCreateSecurityContext', # 0x1a
    'NtAlpcDeletePortSection', # 0x1b
    'NtAlpcDeleteResourceReserve', # 0x1c
    'NtAlpcDeleteSectionView', # 0x1d
    'NtAlpcDeleteSecurityContext', # 0x1e
    'NtAlpcDisconnectPort', # 0x1f
    'NtAlpcImpersonateClientOfPort', # 0x20
    'NtAlpcOpenSenderProcess', # 0x21
    'NtAlpcOpenSenderThread', # 0x22
    'NtAlpcQueryInformation', # 0x23
    'NtAlpcQueryInformationMessage', # 0x24
    'NtAlpcRevokeSecurityContext', # 0x25
    'NtAlpcSendWaitReceivePort', # 0x26
    'NtAlpcSetInformation', # 0x27
    'NtApphelpCacheControl', # 0x28
    'NtAreMappedFilesTheSame', # 0x29
    'NtAssignProcessToJobObject', # 0x2a
    'NtCallbackReturn', # 0x2b
    'NtCancelDeviceWakeupRequest', # 0x2c
    'NtCancelIoFile', # 0x2d
    'NtCancelTimer', # 0x2e
    'NtClearEvent', # 0x2f
    'NtClose', # 0x30
    'NtCloseObjectAuditAlarm', # 0x31
    'NtCompactKeys', # 0x32
    'NtCompareTokens', # 0x33
    'NtCompleteConnectPort', # 0x34
    'NtCompressKey', # 0x35
    'NtConnectPort', # 0x36
    'NtContinue', # 0x37
    'NtCreateDebugObject', # 0x38
    'NtCreateDirectoryObject', # 0x39
    'NtCreateEvent', # 0x3a
    'NtCreateEventPair', # 0x3b
    'NtCreateFile', # 0x3c
    'NtCreateIoCompletion', # 0x3d
    'NtCreateJobObject', # 0x3e
    'NtCreateJobSet', # 0x3f
    'NtCreateKey', # 0x40
    'NtCreateKeyTransacted', # 0x41
    'NtCreateMailslotFile', # 0x42
    'NtCreateMutant', # 0x43
    'NtCreateNamedPipeFile', # 0x44
    'NtCreatePrivateNamespace', # 0x45
    'NtCreatePagingFile', # 0x46
    'NtCreatePort', # 0x47
    'NtCreateProcess', # 0x48
    'NtCreateProcessEx', # 0x49
    'NtCreateProfile', # 0x4a
    'NtCreateSection', # 0x4b
    'NtCreateSemaphore', # 0x4c
    'NtCreateSymbolicLinkObject', # 0x4d
    'NtCreateThread', # 0x4e
    'NtCreateTimer', # 0x4f
    'NtCreateToken', # 0x50
    'NtCreateTransaction', # 0x51
    'NtOpenTransaction', # 0x52
    'NtQueryInformationTransaction', # 0x53
    'NtQueryInformationTransactionManager', # 0x54
    'NtPrePrepareEnlistment', # 0x55
    'NtPrepareEnlistment', # 0x56
    'NtCommitEnlistment', # 0x57
    'NtReadOnlyEnlistment', # 0x58
    'NtRollbackComplete', # 0x59
    'NtRollbackEnlistment', # 0x5a
    'NtCommitTransaction', # 0x5b
    'NtRollbackTransaction', # 0x5c
    'NtPrePrepareComplete', # 0x5d
    'NtPrepareComplete', # 0x5e
    'NtCommitComplete', # 0x5f
    'NtSinglePhaseReject', # 0x60
    'NtSetInformationTransaction', # 0x61
    'NtSetInformationTransactionManager', # 0x62
    'NtSetInformationResourceManager', # 0x63
    'NtCreateTransactionManager', # 0x64
    'NtOpenTransactionManager', # 0x65
    'NtRollforwardTransactionManager', # 0x66
    'NtRecoverEnlistment', # 0x67
    'NtRecoverResourceManager', # 0x68
    'NtRecoverTransactionManager', # 0x69
    'NtCreateResourceManager', # 0x6a
    'NtOpenResourceManager', # 0x6b
    'NtGetNotificationResourceManager', # 0x6c
    'NtQueryInformationResourceManager', # 0x6d
    'NtCreateEnlistment', # 0x6e
    'NtOpenEnlistment', # 0x6f
    'NtSetInformationEnlistment', # 0x70
    'NtQueryInformationEnlistment', # 0x71
    'NtStartTm', # 0x72
    'NtCreateWaitablePort', # 0x73
    'NtDebugActiveProcess', # 0x74
    'NtDebugContinue', # 0x75
    'NtDelayExecution', # 0x76
    'NtDeleteAtom', # 0x77
    'NtDeleteBootEntry', # 0x78
    'NtDeleteDriverEntry', # 0x79
    'NtDeleteFile', # 0x7a
    'NtDeleteKey', # 0x7b
    'NtDeletePrivateNamespace', # 0x7c
    'NtDeleteObjectAuditAlarm', # 0x7d
    'NtDeleteValueKey', # 0x7e
    'NtDeviceIoControlFile', # 0x7f
    'NtDisplayString', # 0x80
    'NtDuplicateObject', # 0x81
    'NtDuplicateToken', # 0x82
    'NtEnumerateBootEntries', # 0x83
    'NtEnumerateDriverEntries', # 0x84
    'NtEnumerateKey', # 0x85
    'NtEnumerateSystemEnvironmentValuesEx', # 0x86
    'NtEnumerateTransactionObject', # 0x87
    'NtEnumerateValueKey', # 0x88
    'NtExtendSection', # 0x89
    'NtFilterToken', # 0x8a
    'NtFindAtom', # 0x8b
    'NtFlushBuffersFile', # 0x8c
    'NtFlushInstructionCache', # 0x8d
    'NtFlushKey', # 0x8e
    'NtFlushProcessWriteBuffers', # 0x8f
    'NtFlushVirtualMemory', # 0x90
    'NtFlushWriteBuffer', # 0x91
    'NtFreeUserPhysicalPages', # 0x92
    'NtFreeVirtualMemory', # 0x93
    'NtFreezeRegistry', # 0x94
    'NtFreezeTransactions', # 0x95
    'NtFsControlFile', # 0x96
    'NtGetContextThread', # 0x97
    'NtGetDevicePowerState', # 0x98
    'NtGetNlsSectionPtr', # 0x99
    'NtGetPlugPlayEvent', # 0x9a
    'NtGetWriteWatch', # 0x9b
    'NtImpersonateAnonymousToken', # 0x9c
    'NtImpersonateClientOfPort', # 0x9d
    'NtImpersonateThread', # 0x9e
    'NtInitializeNlsFiles', # 0x9f
    'NtInitializeRegistry', # 0xa0
    'NtInitiatePowerAction', # 0xa1
    'NtIsProcessInJob', # 0xa2
    'NtIsSystemResumeAutomatic', # 0xa3
    'NtListenPort', # 0xa4
    'NtLoadDriver', # 0xa5
    'NtLoadKey', # 0xa6
    'NtLoadKey2', # 0xa7
    'NtLoadKeyEx', # 0xa8
    'NtLockFile', # 0xa9
    'NtLockProductActivationKeys', # 0xaa
    'NtLockRegistryKey', # 0xab
    'NtLockVirtualMemory', # 0xac
    'NtMakePermanentObject', # 0xad
    'NtMakeTemporaryObject', # 0xae
    'NtMapUserPhysicalPages', # 0xaf
    'NtMapUserPhysicalPagesScatter', # 0xb0
    'NtMapViewOfSection', # 0xb1
    'NtModifyBootEntry', # 0xb2
    'NtModifyDriverEntry', # 0xb3
    'NtNotifyChangeDirectoryFile', # 0xb4
    'NtNotifyChangeKey', # 0xb5
    'NtNotifyChangeMultipleKeys', # 0xb6
    'NtOpenDirectoryObject', # 0xb7
    'NtOpenEvent', # 0xb8
    'NtOpenEventPair', # 0xb9
    'NtOpenFile', # 0xba
    'NtOpenIoCompletion', # 0xbb
    'NtOpenJobObject', # 0xbc
    'NtOpenKey', # 0xbd
    'NtOpenKeyTransacted', # 0xbe
    'NtOpenMutant', # 0xbf
    'NtOpenPrivateNamespace', # 0xc0
    'NtOpenObjectAuditAlarm', # 0xc1
    'NtOpenProcess', # 0xc2
    'NtOpenProcessToken', # 0xc3
    'NtOpenProcessTokenEx', # 0xc4
    'NtOpenSection', # 0xc5
    'NtOpenSemaphore', # 0xc6
    'NtOpenSession', # 0xc7
    'NtOpenSymbolicLinkObject', # 0xc8
    'NtOpenThread', # 0xc9
    'NtOpenThreadToken', # 0xca
    'NtOpenThreadTokenEx', # 0xcb
    'NtOpenTimer', # 0xcc
    'NtPlugPlayControl', # 0xcd
    'NtPowerInformation', # 0xce
    'NtPrivilegeCheck', # 0xcf
    'NtPrivilegeObjectAuditAlarm', # 0xd0
    'NtPrivilegedServiceAuditAlarm', # 0xd1
    'NtProtectVirtualMemory', # 0xd2
    'NtPulseEvent', # 0xd3
    'NtQueryAttributesFile', # 0xd4
    'NtQueryBootEntryOrder', # 0xd5
    'NtQueryBootOptions', # 0xd6
    'NtQueryDebugFilterState', # 0xd7
    'NtQueryDefaultLocale', # 0xd8
    'NtQueryDefaultUILanguage', # 0xd9
    'NtQueryDirectoryFile', # 0xda
    'NtQueryDirectoryObject', # 0xdb
    'NtQueryDriverEntryOrder', # 0xdc
    'NtQueryEaFile', # 0xdd
    'NtQueryEvent', # 0xde
    'NtQueryFullAttributesFile', # 0xdf
    'NtQueryInformationAtom', # 0xe0
    'NtQueryInformationFile', # 0xe1
    'NtQueryInformationJobObject', # 0xe2
    'NtQueryInformationPort', # 0xe3
    'NtQueryInformationProcess', # 0xe4
    'NtQueryInformationThread', # 0xe5
    'NtQueryInformationToken', # 0xe6
    'NtQueryInstallUILanguage', # 0xe7
    'NtQueryIntervalProfile', # 0xe8
    'NtQueryIoCompletion', # 0xe9
    'NtQueryKey', # 0xea
    'NtQueryMultipleValueKey', # 0xeb
    'NtQueryMutant', # 0xec
    'NtQueryObject', # 0xed
    'NtQueryOpenSubKeys', # 0xee
    'NtQueryOpenSubKeysEx', # 0xef
    'NtQueryPerformanceCounter', # 0xf0
    'NtQueryQuotaInformationFile', # 0xf1
    'NtQuerySection', # 0xf2
    'NtQuerySecurityObject', # 0xf3
    'NtQuerySemaphore', # 0xf4
    'NtQuerySymbolicLinkObject', # 0xf5
    'NtQuerySystemEnvironmentValue', # 0xf6
    'NtQuerySystemEnvironmentValueEx', # 0xf7
    'NtQuerySystemInformation', # 0xf8
    'NtQuerySystemTime', # 0xf9
    'NtQueryTimer', # 0xfa
    'NtQueryTimerResolution', # 0xfb
    'NtQueryValueKey', # 0xfc
    'NtQueryVirtualMemory', # 0xfd
    'NtQueryVolumeInformationFile', # 0xfe
    'NtQueueApcThread', # 0xff
    'NtRaiseException', # 0x100
    'NtRaiseHardError', # 0x101
    'NtReadFile', # 0x102
    'NtReadFileScatter', # 0x103
    'NtReadRequestData', # 0x104
    'NtReadVirtualMemory', # 0x105
    'NtRegisterThreadTerminatePort', # 0x106
    'NtReleaseMutant', # 0x107
    'NtReleaseSemaphore', # 0x108
    'NtRemoveIoCompletion', # 0x109
    'NtRemoveProcessDebug', # 0x10a
    'NtRenameKey', # 0x10b
    'NtReplaceKey', # 0x10c
    'NtReplyPort', # 0x10d
    'NtReplyWaitReceivePort', # 0x10e
    'NtReplyWaitReceivePortEx', # 0x10f
    'NtReplyWaitReplyPort', # 0x110
    'NtRequestDeviceWakeup', # 0x111
    'NtRequestPort', # 0x112
    'NtRequestWaitReplyPort', # 0x113
    'NtRequestWakeupLatency', # 0x114
    'NtResetEvent', # 0x115
    'NtResetWriteWatch', # 0x116
    'NtRestoreKey', # 0x117
    'NtResumeProcess', # 0x118
    'NtResumeThread', # 0x119
    'NtSaveKey', # 0x11a
    'NtSaveKeyEx', # 0x11b
    'NtSaveMergedKeys', # 0x11c
    'NtClearSavepointTransaction', # 0x11d
    'NtClearAllSavepointsTransaction', # 0x11e
    'NtRollbackSavepointTransaction', # 0x11f
    'NtSavepointTransaction', # 0x120
    'NtSavepointComplete', # 0x121
    'NtSecureConnectPort', # 0x122
    'NtSetBootEntryOrder', # 0x123
    'NtSetBootOptions', # 0x124
    'NtSetContextThread', # 0x125
    'NtSetDebugFilterState', # 0x126
    'NtSetDefaultHardErrorPort', # 0x127
    'NtSetDefaultLocale', # 0x128
    'NtSetDefaultUILanguage', # 0x129
    'NtSetDriverEntryOrder', # 0x12a
    'NtSetEaFile', # 0x12b
    'NtSetEvent', # 0x12c
    'NtSetEventBoostPriority', # 0x12d
    'NtSetHighEventPair', # 0x12e
    'NtSetHighWaitLowEventPair', # 0x12f
    'NtSetInformationDebugObject', # 0x130
    'NtSetInformationFile', # 0x131
    'NtSetInformationJobObject', # 0x132
    'NtSetInformationKey', # 0x133
    'NtSetInformationObject', # 0x134
    'NtSetInformationProcess', # 0x135
    'NtSetInformationThread', # 0x136
    'NtSetInformationToken', # 0x137
    'NtSetIntervalProfile', # 0x138
    'NtSetIoCompletion', # 0x139
    'NtSetLdtEntries', # 0x13a
    'NtSetLowEventPair', # 0x13b
    'NtSetLowWaitHighEventPair', # 0x13c
    'NtSetQuotaInformationFile', # 0x13d
    'NtSetSecurityObject', # 0x13e
    'NtSetSystemEnvironmentValue', # 0x13f
    'NtSetSystemEnvironmentValueEx', # 0x140
    'NtSetSystemInformation', # 0x141
    'NtSetSystemPowerState', # 0x142
    'NtSetSystemTime', # 0x143
    'NtSetThreadExecutionState', # 0x144
    'NtSetTimer', # 0x145
    'NtSetTimerResolution', # 0x146
    'NtSetUuidSeed', # 0x147
    'NtSetValueKey', # 0x148
    'NtSetVolumeInformationFile', # 0x149
    'NtShutdownSystem', # 0x14a
    'NtSignalAndWaitForSingleObject', # 0x14b
    'NtStartProfile', # 0x14c
    'NtStopProfile', # 0x14d
    'NtSuspendProcess', # 0x14e
    'NtSuspendThread', # 0x14f
    'NtSystemDebugControl', # 0x150
    'NtTerminateJobObject', # 0x151
    'NtTerminateProcess', # 0x152
    'NtTerminateThread', # 0x153
    'NtTestAlert', # 0x154
    'NtThawRegistry', # 0x155
    'NtThawTransactions', # 0x156
    'NtTraceEvent', # 0x157
    'NtTraceControl', # 0x158
    'NtTranslateFilePath', # 0x159
    'NtUnloadDriver', # 0x15a
    'NtUnloadKey', # 0x15b
    'NtUnloadKey2', # 0x15c
    'NtUnloadKeyEx', # 0x15d
    'NtUnlockFile', # 0x15e
    'NtUnlockVirtualMemory', # 0x15f
    'NtUnmapViewOfSection', # 0x160
    'NtVdmControl', # 0x161
    'NtWaitForDebugEvent', # 0x162
    'NtWaitForMultipleObjects', # 0x163
    'NtWaitForSingleObject', # 0x164
    'NtWaitHighEventPair', # 0x165
    'NtWaitLowEventPair', # 0x166
    'NtWriteFile', # 0x167
    'NtWriteFileGather', # 0x168
    'NtWriteRequestData', # 0x169
    'NtWriteVirtualMemory', # 0x16a
    'NtYieldExecution', # 0x16b
    'NtCreateKeyedEvent', # 0x16c
    'NtOpenKeyedEvent', # 0x16d
    'NtReleaseKeyedEvent', # 0x16e
    'NtWaitForKeyedEvent', # 0x16f
    'NtQueryPortInformationProcess', # 0x170
    'NtGetCurrentProcessorNumber', # 0x171
    'NtWaitForMultipleObjects32', # 0x172
    'NtGetNextProcess', # 0x173
    'NtGetNextThread', # 0x174
    'NtCancelIoFileEx', # 0x175
    'NtCancelSynchronousIoFile', # 0x176
    'NtRemoveIoCompletionEx', # 0x177
    'NtRegisterProtocolAddressInformation', # 0x178
    'NtPullTransaction', # 0x179
    'NtMarshallTransaction', # 0x17a
    'NtPropagationComplete', # 0x17b
    'NtPropagationFailed', # 0x17c
    'NtCreateWorkerFactory', # 0x17d
    'NtReleaseWorkerFactoryWorker', # 0x17e
    'NtWaitForWorkViaWorkerFactory', # 0x17f
    'NtSetInformationWorkerFactory', # 0x180
    'NtQueryInformationWorkerFactory', # 0x181
    'NtWorkerFactoryWorkerReady', # 0x182
    'NtShutdownWorkerFactory', # 0x183
    'NtCreateThreadEx', # 0x184
    'NtCreateUserProcess', # 0x185
    'NtQueryLicenseValue', # 0x186
    'NtMapCMFModule', # 0x187
    'NtListTransactions', # 0x188
    'NtIsUILanguageComitted', # 0x189
    'NtFlushInstallUILanguage', # 0x18a
    'NtGetMUIRegistryInfo', # 0x18b
    'NtAcquireCMFViewOwnership', # 0x18c
    'NtReleaseCMFViewOwnership', # 0x18d
    ],
    [
    'NtGdiAbortDoc', # 0x0
    'NtGdiAbortPath', # 0x1
    'NtGdiAddFontResourceW', # 0x2
    'NtGdiAddRemoteFontToDC', # 0x3
    'NtGdiAddFontMemResourceEx', # 0x4
    'NtGdiRemoveMergeFont', # 0x5
    'NtGdiAddRemoteMMInstanceToDC', # 0x6
    'NtGdiAlphaBlend', # 0x7
    'NtGdiAngleArc', # 0x8
    'NtGdiAnyLinkedFonts', # 0x9
    'NtGdiFontIsLinked', # 0xa
    'NtGdiArcInternal', # 0xb
    'NtGdiBeginPath', # 0xc
    'NtGdiBitBlt', # 0xd
    'NtGdiCancelDC', # 0xe
    'NtGdiCheckBitmapBits', # 0xf
    'NtGdiCloseFigure', # 0x10
    'NtGdiClearBitmapAttributes', # 0x11
    'NtGdiClearBrushAttributes', # 0x12
    'NtGdiColorCorrectPalette', # 0x13
    'NtGdiCombineRgn', # 0x14
    'NtGdiCombineTransform', # 0x15
    'NtGdiComputeXformCoefficients', # 0x16
    'NtGdiConfigureOPMProtectedOutput', # 0x17
    'NtGdiConsoleTextOut', # 0x18
    'NtGdiConvertMetafileRect', # 0x19
    'NtGdiCreateBitmap', # 0x1a
    'NtGdiCreateClientObj', # 0x1b
    'NtGdiCreateColorSpace', # 0x1c
    'NtGdiCreateColorTransform', # 0x1d
    'NtGdiCreateCompatibleBitmap', # 0x1e
    'NtGdiCreateCompatibleDC', # 0x1f
    'NtGdiCreateDIBBrush', # 0x20
    'NtGdiCreateDIBitmapInternal', # 0x21
    'NtGdiCreateDIBSection', # 0x22
    'NtGdiCreateEllipticRgn', # 0x23
    'NtGdiCreateHalftonePalette', # 0x24
    'NtGdiCreateHatchBrushInternal', # 0x25
    'NtGdiCreateMetafileDC', # 0x26
    'NtGdiCreateOPMProtectedOutputs', # 0x27
    'NtGdiCreatePaletteInternal', # 0x28
    'NtGdiCreatePatternBrushInternal', # 0x29
    'NtGdiCreatePen', # 0x2a
    'NtGdiCreateRectRgn', # 0x2b
    'NtGdiCreateRoundRectRgn', # 0x2c
    'NtGdiCreateServerMetaFile', # 0x2d
    'NtGdiCreateSolidBrush', # 0x2e
    'NtGdiD3dContextCreate', # 0x2f
    'NtGdiD3dContextDestroy', # 0x30
    'NtGdiD3dContextDestroyAll', # 0x31
    'NtGdiD3dValidateTextureStageState', # 0x32
    'NtGdiD3dDrawPrimitives2', # 0x33
    'NtGdiDdGetDriverState', # 0x34
    'NtGdiDdAddAttachedSurface', # 0x35
    'NtGdiDdAlphaBlt', # 0x36
    'NtGdiDdAttachSurface', # 0x37
    'NtGdiDdBeginMoCompFrame', # 0x38
    'NtGdiDdBlt', # 0x39
    'NtGdiDdCanCreateSurface', # 0x3a
    'NtGdiDdCanCreateD3DBuffer', # 0x3b
    'NtGdiDdColorControl', # 0x3c
    'NtGdiDdCreateDirectDrawObject', # 0x3d
    'NtGdiDdCreateSurface', # 0x3e
    'NtGdiDdCreateD3DBuffer', # 0x3f
    'NtGdiDdCreateMoComp', # 0x40
    'NtGdiDdCreateSurfaceObject', # 0x41
    'NtGdiDdDeleteDirectDrawObject', # 0x42
    'NtGdiDdDeleteSurfaceObject', # 0x43
    'NtGdiDdDestroyMoComp', # 0x44
    'NtGdiDdDestroySurface', # 0x45
    'NtGdiDdDestroyD3DBuffer', # 0x46
    'NtGdiDdEndMoCompFrame', # 0x47
    'NtGdiDdFlip', # 0x48
    'NtGdiDdFlipToGDISurface', # 0x49
    'NtGdiDdGetAvailDriverMemory', # 0x4a
    'NtGdiDdGetBltStatus', # 0x4b
    'NtGdiDdGetDC', # 0x4c
    'NtGdiDdGetDriverInfo', # 0x4d
    'NtGdiDdGetDxHandle', # 0x4e
    'NtGdiDdGetFlipStatus', # 0x4f
    'NtGdiDdGetInternalMoCompInfo', # 0x50
    'NtGdiDdGetMoCompBuffInfo', # 0x51
    'NtGdiDdGetMoCompGuids', # 0x52
    'NtGdiDdGetMoCompFormats', # 0x53
    'NtGdiDdGetScanLine', # 0x54
    'NtGdiDdLock', # 0x55
    'NtGdiDdLockD3D', # 0x56
    'NtGdiDdQueryDirectDrawObject', # 0x57
    'NtGdiDdQueryMoCompStatus', # 0x58
    'NtGdiDdReenableDirectDrawObject', # 0x59
    'NtGdiDdReleaseDC', # 0x5a
    'NtGdiDdRenderMoComp', # 0x5b
    'NtGdiDdResetVisrgn', # 0x5c
    'NtGdiDdSetColorKey', # 0x5d
    'NtGdiDdSetExclusiveMode', # 0x5e
    'NtGdiDdSetGammaRamp', # 0x5f
    'NtGdiDdCreateSurfaceEx', # 0x60
    'NtGdiDdSetOverlayPosition', # 0x61
    'NtGdiDdUnattachSurface', # 0x62
    'NtGdiDdUnlock', # 0x63
    'NtGdiDdUnlockD3D', # 0x64
    'NtGdiDdUpdateOverlay', # 0x65
    'NtGdiDdWaitForVerticalBlank', # 0x66
    'NtGdiDvpCanCreateVideoPort', # 0x67
    'NtGdiDvpColorControl', # 0x68
    'NtGdiDvpCreateVideoPort', # 0x69
    'NtGdiDvpDestroyVideoPort', # 0x6a
    'NtGdiDvpFlipVideoPort', # 0x6b
    'NtGdiDvpGetVideoPortBandwidth', # 0x6c
    'NtGdiDvpGetVideoPortField', # 0x6d
    'NtGdiDvpGetVideoPortFlipStatus', # 0x6e
    'NtGdiDvpGetVideoPortInputFormats', # 0x6f
    'NtGdiDvpGetVideoPortLine', # 0x70
    'NtGdiDvpGetVideoPortOutputFormats', # 0x71
    'NtGdiDvpGetVideoPortConnectInfo', # 0x72
    'NtGdiDvpGetVideoSignalStatus', # 0x73
    'NtGdiDvpUpdateVideoPort', # 0x74
    'NtGdiDvpWaitForVideoPortSync', # 0x75
    'NtGdiDvpAcquireNotification', # 0x76
    'NtGdiDvpReleaseNotification', # 0x77
    'NtGdiDxgGenericThunk', # 0x78
    'NtGdiDeleteClientObj', # 0x79
    'NtGdiDeleteColorSpace', # 0x7a
    'NtGdiDeleteColorTransform', # 0x7b
    'NtGdiDeleteObjectApp', # 0x7c
    'NtGdiDescribePixelFormat', # 0x7d
    'NtGdiDestroyOPMProtectedOutput', # 0x7e
    'NtGdiGetPerBandInfo', # 0x7f
    'NtGdiDoBanding', # 0x80
    'NtGdiDoPalette', # 0x81
    'NtGdiDrawEscape', # 0x82
    'NtGdiEllipse', # 0x83
    'NtGdiEnableEudc', # 0x84
    'NtGdiEndDoc', # 0x85
    'NtGdiEndPage', # 0x86
    'NtGdiEndPath', # 0x87
    'NtGdiEnumFontChunk', # 0x88
    'NtGdiEnumFontClose', # 0x89
    'NtGdiEnumFontOpen', # 0x8a
    'NtGdiEnumObjects', # 0x8b
    'NtGdiEqualRgn', # 0x8c
    'NtGdiEudcLoadUnloadLink', # 0x8d
    'NtGdiExcludeClipRect', # 0x8e
    'NtGdiExtCreatePen', # 0x8f
    'NtGdiExtCreateRegion', # 0x90
    'NtGdiExtEscape', # 0x91
    'NtGdiExtFloodFill', # 0x92
    'NtGdiExtGetObjectW', # 0x93
    'NtGdiExtSelectClipRgn', # 0x94
    'NtGdiExtTextOutW', # 0x95
    'NtGdiFillPath', # 0x96
    'NtGdiFillRgn', # 0x97
    'NtGdiFlattenPath', # 0x98
    'NtGdiFlush', # 0x99
    'NtGdiForceUFIMapping', # 0x9a
    'NtGdiFrameRgn', # 0x9b
    'NtGdiFullscreenControl', # 0x9c
    'NtGdiGetAndSetDCDword', # 0x9d
    'NtGdiGetAppClipBox', # 0x9e
    'NtGdiGetBitmapBits', # 0x9f
    'NtGdiGetBitmapDimension', # 0xa0
    'NtGdiGetBoundsRect', # 0xa1
    'NtGdiGetCertificate', # 0xa2
    'NtGdiGetCertificateSize', # 0xa3
    'NtGdiGetCharABCWidthsW', # 0xa4
    'NtGdiGetCharacterPlacementW', # 0xa5
    'NtGdiGetCharSet', # 0xa6
    'NtGdiGetCharWidthW', # 0xa7
    'NtGdiGetCharWidthInfo', # 0xa8
    'NtGdiGetColorAdjustment', # 0xa9
    'NtGdiGetColorSpaceforBitmap', # 0xaa
    'NtGdiGetCOPPCompatibleOPMInformation', # 0xab
    'NtGdiGetDCDword', # 0xac
    'NtGdiGetDCforBitmap', # 0xad
    'NtGdiGetDCObject', # 0xae
    'NtGdiGetDCPoint', # 0xaf
    'NtGdiGetDeviceCaps', # 0xb0
    'NtGdiGetDeviceGammaRamp', # 0xb1
    'NtGdiGetDeviceCapsAll', # 0xb2
    'NtGdiGetDIBitsInternal', # 0xb3
    'NtGdiGetETM', # 0xb4
    'NtGdiGetEudcTimeStampEx', # 0xb5
    'NtGdiGetFontData', # 0xb6
    'NtGdiGetFontResourceInfoInternalW', # 0xb7
    'NtGdiGetGlyphIndicesW', # 0xb8
    'NtGdiGetGlyphIndicesWInternal', # 0xb9
    'NtGdiGetGlyphOutline', # 0xba
    'NtGdiGetOPMInformation', # 0xbb
    'NtGdiGetKerningPairs', # 0xbc
    'NtGdiGetLinkedUFIs', # 0xbd
    'NtGdiGetMiterLimit', # 0xbe
    'NtGdiGetMonitorID', # 0xbf
    'NtGdiGetNearestColor', # 0xc0
    'NtGdiGetNearestPaletteIndex', # 0xc1
    'NtGdiGetObjectBitmapHandle', # 0xc2
    'NtGdiGetOPMRandomNumber', # 0xc3
    'NtGdiGetOutlineTextMetricsInternalW', # 0xc4
    'NtGdiGetPath', # 0xc5
    'NtGdiGetPixel', # 0xc6
    'NtGdiGetRandomRgn', # 0xc7
    'NtGdiGetRasterizerCaps', # 0xc8
    'NtGdiGetRealizationInfo', # 0xc9
    'NtGdiGetRegionData', # 0xca
    'NtGdiGetRgnBox', # 0xcb
    'NtGdiGetServerMetaFileBits', # 0xcc
    'NtGdiGetSpoolMessage', # 0xcd
    'NtGdiGetStats', # 0xce
    'NtGdiGetStockObject', # 0xcf
    'NtGdiGetStringBitmapW', # 0xd0
    'NtGdiGetSuggestedOPMProtectedOutputArraySize', # 0xd1
    'NtGdiGetSystemPaletteUse', # 0xd2
    'NtGdiGetTextCharsetInfo', # 0xd3
    'NtGdiGetTextExtent', # 0xd4
    'NtGdiGetTextExtentExW', # 0xd5
    'NtGdiGetTextFaceW', # 0xd6
    'NtGdiGetTextMetricsW', # 0xd7
    'NtGdiGetTransform', # 0xd8
    'NtGdiGetUFI', # 0xd9
    'NtGdiGetEmbUFI', # 0xda
    'NtGdiGetUFIPathname', # 0xdb
    'NtGdiGetEmbedFonts', # 0xdc
    'NtGdiChangeGhostFont', # 0xdd
    'NtGdiAddEmbFontToDC', # 0xde
    'NtGdiGetFontUnicodeRanges', # 0xdf
    'NtGdiGetWidthTable', # 0xe0
    'NtGdiGradientFill', # 0xe1
    'NtGdiHfontCreate', # 0xe2
    'NtGdiIcmBrushInfo', # 0xe3
    'NtGdiInit', # 0xe4
    'NtGdiInitSpool', # 0xe5
    'NtGdiIntersectClipRect', # 0xe6
    'NtGdiInvertRgn', # 0xe7
    'NtGdiLineTo', # 0xe8
    'NtGdiMakeFontDir', # 0xe9
    'NtGdiMakeInfoDC', # 0xea
    'NtGdiMaskBlt', # 0xeb
    'NtGdiModifyWorldTransform', # 0xec
    'NtGdiMonoBitmap', # 0xed
    'NtGdiMoveTo', # 0xee
    'NtGdiOffsetClipRgn', # 0xef
    'NtGdiOffsetRgn', # 0xf0
    'NtGdiOpenDCW', # 0xf1
    'NtGdiPatBlt', # 0xf2
    'NtGdiPolyPatBlt', # 0xf3
    'NtGdiPathToRegion', # 0xf4
    'NtGdiPlgBlt', # 0xf5
    'NtGdiPolyDraw', # 0xf6
    'NtGdiPolyPolyDraw', # 0xf7
    'NtGdiPolyTextOutW', # 0xf8
    'NtGdiPtInRegion', # 0xf9
    'NtGdiPtVisible', # 0xfa
    'NtGdiQueryFonts', # 0xfb
    'NtGdiQueryFontAssocInfo', # 0xfc
    'NtGdiRectangle', # 0xfd
    'NtGdiRectInRegion', # 0xfe
    'NtGdiRectVisible', # 0xff
    'NtGdiRemoveFontResourceW', # 0x100
    'NtGdiRemoveFontMemResourceEx', # 0x101
    'NtGdiResetDC', # 0x102
    'NtGdiResizePalette', # 0x103
    'NtGdiRestoreDC', # 0x104
    'NtGdiRoundRect', # 0x105
    'NtGdiSaveDC', # 0x106
    'NtGdiScaleViewportExtEx', # 0x107
    'NtGdiScaleWindowExtEx', # 0x108
    'NtGdiSelectBitmap', # 0x109
    'NtGdiSelectBrush', # 0x10a
    'NtGdiSelectClipPath', # 0x10b
    'NtGdiSelectFont', # 0x10c
    'NtGdiSelectPen', # 0x10d
    'NtGdiSetBitmapAttributes', # 0x10e
    'NtGdiSetBitmapBits', # 0x10f
    'NtGdiSetBitmapDimension', # 0x110
    'NtGdiSetBoundsRect', # 0x111
    'NtGdiSetBrushAttributes', # 0x112
    'NtGdiSetBrushOrg', # 0x113
    'NtGdiSetColorAdjustment', # 0x114
    'NtGdiSetColorSpace', # 0x115
    'NtGdiSetDeviceGammaRamp', # 0x116
    'NtGdiSetDIBitsToDeviceInternal', # 0x117
    'NtGdiSetFontEnumeration', # 0x118
    'NtGdiSetFontXform', # 0x119
    'NtGdiSetIcmMode', # 0x11a
    'NtGdiSetLinkedUFIs', # 0x11b
    'NtGdiSetMagicColors', # 0x11c
    'NtGdiSetMetaRgn', # 0x11d
    'NtGdiSetMiterLimit', # 0x11e
    'NtGdiGetDeviceWidth', # 0x11f
    'NtGdiMirrorWindowOrg', # 0x120
    'NtGdiSetLayout', # 0x121
    'NtGdiSetOPMSigningKeyAndSequenceNumbers', # 0x122
    'NtGdiSetPixel', # 0x123
    'NtGdiSetPixelFormat', # 0x124
    'NtGdiSetRectRgn', # 0x125
    'NtGdiSetSystemPaletteUse', # 0x126
    'NtGdiSetTextJustification', # 0x127
    'NtGdiSetupPublicCFONT', # 0x128
    'NtGdiSetVirtualResolution', # 0x129
    'NtGdiSetSizeDevice', # 0x12a
    'NtGdiStartDoc', # 0x12b
    'NtGdiStartPage', # 0x12c
    'NtGdiStretchBlt', # 0x12d
    'NtGdiStretchDIBitsInternal', # 0x12e
    'NtGdiStrokeAndFillPath', # 0x12f
    'NtGdiStrokePath', # 0x130
    'NtGdiSwapBuffers', # 0x131
    'NtGdiTransformPoints', # 0x132
    'NtGdiTransparentBlt', # 0x133
    'NtGdiUnloadPrinterDriver', # 0x134
    'NtGdiUnmapMemFont', # 0x135
    'NtGdiUnrealizeObject', # 0x136
    'NtGdiUpdateColors', # 0x137
    'NtGdiWidenPath', # 0x138
    'NtUserActivateKeyboardLayout', # 0x139
    'NtUserAddClipboardFormatListener', # 0x13a
    'NtUserAlterWindowStyle', # 0x13b
    'NtUserAssociateInputContext', # 0x13c
    'NtUserAttachThreadInput', # 0x13d
    'NtUserBeginPaint', # 0x13e
    'NtUserBitBltSysBmp', # 0x13f
    'NtUserBlockInput', # 0x140
    'NtUserBuildHimcList', # 0x141
    'NtUserBuildHwndList', # 0x142
    'NtUserBuildNameList', # 0x143
    'NtUserBuildPropList', # 0x144
    'NtUserCallHwnd', # 0x145
    'NtUserCallHwndLock', # 0x146
    'NtUserCallHwndOpt', # 0x147
    'NtUserCallHwndParam', # 0x148
    'NtUserCallHwndParamLock', # 0x149
    'NtUserCallMsgFilter', # 0x14a
    'NtUserCallNextHookEx', # 0x14b
    'NtUserCallNoParam', # 0x14c
    'NtUserCallOneParam', # 0x14d
    'NtUserCallTwoParam', # 0x14e
    'NtUserChangeClipboardChain', # 0x14f
    'NtUserChangeDisplaySettings', # 0x150
    'NtUserCheckAccessForIntegrityLevel', # 0x151
    'NtUserCheckDesktopByThreadId', # 0x152
    'NtUserCheckWindowThreadDesktop', # 0x153
    'NtUserCheckImeHotKey', # 0x154
    'NtUserCheckMenuItem', # 0x155
    'NtUserChildWindowFromPointEx', # 0x156
    'NtUserClipCursor', # 0x157
    'NtUserCloseClipboard', # 0x158
    'NtUserCloseDesktop', # 0x159
    'NtUserCloseWindowStation', # 0x15a
    'NtUserConsoleControl', # 0x15b
    'NtUserConvertMemHandle', # 0x15c
    'NtUserCopyAcceleratorTable', # 0x15d
    'NtUserCountClipboardFormats', # 0x15e
    'NtUserCreateAcceleratorTable', # 0x15f
    'NtUserCreateCaret', # 0x160
    'NtUserCreateDesktopEx', # 0x161
    'NtUserCreateInputContext', # 0x162
    'NtUserCreateLocalMemHandle', # 0x163
    'NtUserCreateWindowEx', # 0x164
    'NtUserCreateWindowStation', # 0x165
    'NtUserDdeInitialize', # 0x166
    'NtUserDeferWindowPos', # 0x167
    'NtUserDefSetText', # 0x168
    'NtUserDeleteMenu', # 0x169
    'NtUserDestroyAcceleratorTable', # 0x16a
    'NtUserDestroyCursor', # 0x16b
    'NtUserDestroyInputContext', # 0x16c
    'NtUserDestroyMenu', # 0x16d
    'NtUserDestroyWindow', # 0x16e
    'NtUserDisableThreadIme', # 0x16f
    'NtUserDispatchMessage', # 0x170
    'NtUserDoSoundConnect', # 0x171
    'NtUserDoSoundDisconnect', # 0x172
    'NtUserDragDetect', # 0x173
    'NtUserDragObject', # 0x174
    'NtUserDrawAnimatedRects', # 0x175
    'NtUserDrawCaption', # 0x176
    'NtUserDrawCaptionTemp', # 0x177
    'NtUserDrawIconEx', # 0x178
    'NtUserDrawMenuBarTemp', # 0x179
    'NtUserEmptyClipboard', # 0x17a
    'NtUserEnableMenuItem', # 0x17b
    'NtUserEnableScrollBar', # 0x17c
    'NtUserEndDeferWindowPosEx', # 0x17d
    'NtUserEndMenu', # 0x17e
    'NtUserEndPaint', # 0x17f
    'NtUserEnumDisplayDevices', # 0x180
    'NtUserEnumDisplayMonitors', # 0x181
    'NtUserEnumDisplaySettings', # 0x182
    'NtUserEvent', # 0x183
    'NtUserExcludeUpdateRgn', # 0x184
    'NtUserFillWindow', # 0x185
    'NtUserFindExistingCursorIcon', # 0x186
    'NtUserFindWindowEx', # 0x187
    'NtUserFlashWindowEx', # 0x188
    'NtUserFrostCrashedWindow', # 0x189
    'NtUserGetAltTabInfo', # 0x18a
    'NtUserGetAncestor', # 0x18b
    'NtUserGetAppImeLevel', # 0x18c
    'NtUserGetAsyncKeyState', # 0x18d
    'NtUserGetAtomName', # 0x18e
    'NtUserGetCaretBlinkTime', # 0x18f
    'NtUserGetCaretPos', # 0x190
    'NtUserGetClassInfoEx', # 0x191
    'NtUserGetClassName', # 0x192
    'NtUserGetClipboardData', # 0x193
    'NtUserGetClipboardFormatName', # 0x194
    'NtUserGetClipboardOwner', # 0x195
    'NtUserGetClipboardSequenceNumber', # 0x196
    'NtUserGetClipboardViewer', # 0x197
    'NtUserGetClipCursor', # 0x198
    'NtUserGetComboBoxInfo', # 0x199
    'NtUserGetControlBrush', # 0x19a
    'NtUserGetControlColor', # 0x19b
    'NtUserGetCPD', # 0x19c
    'NtUserGetCursorFrameInfo', # 0x19d
    'NtUserGetCursorInfo', # 0x19e
    'NtUserGetDC', # 0x19f
    'NtUserGetDCEx', # 0x1a0
    'NtUserGetDoubleClickTime', # 0x1a1
    'NtUserGetForegroundWindow', # 0x1a2
    'NtUserGetGuiResources', # 0x1a3
    'NtUserGetGUIThreadInfo', # 0x1a4
    'NtUserGetIconInfo', # 0x1a5
    'NtUserGetIconSize', # 0x1a6
    'NtUserGetImeHotKey', # 0x1a7
    'NtUserGetImeInfoEx', # 0x1a8
    'NtUserGetInternalWindowPos', # 0x1a9
    'NtUserGetKeyboardLayoutList', # 0x1aa
    'NtUserGetKeyboardLayoutName', # 0x1ab
    'NtUserGetKeyboardState', # 0x1ac
    'NtUserGetKeyNameText', # 0x1ad
    'NtUserGetKeyState', # 0x1ae
    'NtUserGetListBoxInfo', # 0x1af
    'NtUserGetMenuBarInfo', # 0x1b0
    'NtUserGetMenuIndex', # 0x1b1
    'NtUserGetMenuItemRect', # 0x1b2
    'NtUserGetMessage', # 0x1b3
    'NtUserGetMouseMovePointsEx', # 0x1b4
    'NtUserGetObjectInformation', # 0x1b5
    'NtUserGetOpenClipboardWindow', # 0x1b6
    'NtUserGetPriorityClipboardFormat', # 0x1b7
    'NtUserGetProcessWindowStation', # 0x1b8
    'NtUserGetRawInputBuffer', # 0x1b9
    'NtUserGetRawInputData', # 0x1ba
    'NtUserGetRawInputDeviceInfo', # 0x1bb
    'NtUserGetRawInputDeviceList', # 0x1bc
    'NtUserGetRegisteredRawInputDevices', # 0x1bd
    'NtUserGetScrollBarInfo', # 0x1be
    'NtUserGetSystemMenu', # 0x1bf
    'NtUserGetThreadDesktop', # 0x1c0
    'NtUserGetThreadState', # 0x1c1
    'NtUserGetTitleBarInfo', # 0x1c2
    'NtUserGetUpdatedClipboardFormats', # 0x1c3
    'NtUserGetUpdateRect', # 0x1c4
    'NtUserGetUpdateRgn', # 0x1c5
    'NtUserGetWindowDC', # 0x1c6
    'NtUserGetWindowPlacement', # 0x1c7
    'NtUserGetWOWClass', # 0x1c8
    'NtUserGhostWindowFromHungWindow', # 0x1c9
    'NtUserHardErrorControl', # 0x1ca
    'NtUserHideCaret', # 0x1cb
    'NtUserHiliteMenuItem', # 0x1cc
    'NtUserHungWindowFromGhostWindow', # 0x1cd
    'NtUserImpersonateDdeClientWindow', # 0x1ce
    'NtUserInitialize', # 0x1cf
    'NtUserInitializeClientPfnArrays', # 0x1d0
    'NtUserInitTask', # 0x1d1
    'NtUserInternalGetWindowText', # 0x1d2
    'NtUserInternalGetWindowIcon', # 0x1d3
    'NtUserInvalidateRect', # 0x1d4
    'NtUserInvalidateRgn', # 0x1d5
    'NtUserIsClipboardFormatAvailable', # 0x1d6
    'NtUserKillTimer', # 0x1d7
    'NtUserLoadKeyboardLayoutEx', # 0x1d8
    'NtUserLockWindowStation', # 0x1d9
    'NtUserLockWindowUpdate', # 0x1da
    'NtUserLockWorkStation', # 0x1db
    'NtUserLogicalToPhysicalPoint', # 0x1dc
    'NtUserMapVirtualKeyEx', # 0x1dd
    'NtUserMenuItemFromPoint', # 0x1de
    'NtUserMessageCall', # 0x1df
    'NtUserMinMaximize', # 0x1e0
    'NtUserMNDragLeave', # 0x1e1
    'NtUserMNDragOver', # 0x1e2
    'NtUserModifyUserStartupInfoFlags', # 0x1e3
    'NtUserMoveWindow', # 0x1e4
    'NtUserNotifyIMEStatus', # 0x1e5
    'NtUserNotifyProcessCreate', # 0x1e6
    'NtUserNotifyWinEvent', # 0x1e7
    'NtUserOpenClipboard', # 0x1e8
    'NtUserOpenDesktop', # 0x1e9
    'NtUserOpenInputDesktop', # 0x1ea
    'NtUserOpenThreadDesktop', # 0x1eb
    'NtUserOpenWindowStation', # 0x1ec
    'NtUserPaintDesktop', # 0x1ed
    'NtUserPaintMonitor', # 0x1ee
    'NtUserPeekMessage', # 0x1ef
    'NtUserPhysicalToLogicalPoint', # 0x1f0
    'NtUserPostMessage', # 0x1f1
    'NtUserPostThreadMessage', # 0x1f2
    'NtUserPrintWindow', # 0x1f3
    'NtUserProcessConnect', # 0x1f4
    'NtUserQueryInformationThread', # 0x1f5
    'NtUserQueryInputContext', # 0x1f6
    'NtUserQuerySendMessage', # 0x1f7
    'NtUserQueryWindow', # 0x1f8
    'NtUserRealChildWindowFromPoint', # 0x1f9
    'NtUserRealInternalGetMessage', # 0x1fa
    'NtUserRealWaitMessageEx', # 0x1fb
    'NtUserRedrawWindow', # 0x1fc
    'NtUserRegisterClassExWOW', # 0x1fd
    'NtUserRegisterErrorReportingDialog', # 0x1fe
    'NtUserRegisterUserApiHook', # 0x1ff
    'NtUserRegisterHotKey', # 0x200
    'NtUserRegisterRawInputDevices', # 0x201
    'NtUserRegisterTasklist', # 0x202
    'NtUserRegisterWindowMessage', # 0x203
    'NtUserRemoveClipboardFormatListener', # 0x204
    'NtUserRemoveMenu', # 0x205
    'NtUserRemoveProp', # 0x206
    'NtUserResolveDesktop', # 0x207
    'NtUserResolveDesktopForWOW', # 0x208
    'NtUserSBGetParms', # 0x209
    'NtUserScrollDC', # 0x20a
    'NtUserScrollWindowEx', # 0x20b
    'NtUserSelectPalette', # 0x20c
    'NtUserSendInput', # 0x20d
    'NtUserSetActiveWindow', # 0x20e
    'NtUserSetAppImeLevel', # 0x20f
    'NtUserSetCapture', # 0x210
    'NtUserSetClassLong', # 0x211
    'NtUserSetClassWord', # 0x212
    'NtUserSetClipboardData', # 0x213
    'NtUserSetClipboardViewer', # 0x214
    'NtUserSetConsoleReserveKeys', # 0x215
    'NtUserSetCursor', # 0x216
    'NtUserSetCursorContents', # 0x217
    'NtUserSetCursorIconData', # 0x218
    'NtUserSetFocus', # 0x219
    'NtUserSetImeHotKey', # 0x21a
    'NtUserSetImeInfoEx', # 0x21b
    'NtUserSetImeOwnerWindow', # 0x21c
    'NtUserSetInformationProcess', # 0x21d
    'NtUserSetInformationThread', # 0x21e
    'NtUserSetInternalWindowPos', # 0x21f
    'NtUserSetKeyboardState', # 0x220
    'NtUserSetMenu', # 0x221
    'NtUserSetMenuContextHelpId', # 0x222
    'NtUserSetMenuDefaultItem', # 0x223
    'NtUserSetMenuFlagRtoL', # 0x224
    'NtUserSetObjectInformation', # 0x225
    'NtUserSetParent', # 0x226
    'NtUserSetProcessWindowStation', # 0x227
    'NtUserGetProp', # 0x228
    'NtUserSetProp', # 0x229
    'NtUserSetScrollInfo', # 0x22a
    'NtUserSetShellWindowEx', # 0x22b
    'NtUserSetSysColors', # 0x22c
    'NtUserSetSystemCursor', # 0x22d
    'NtUserSetSystemMenu', # 0x22e
    'NtUserSetSystemTimer', # 0x22f
    'NtUserSetThreadDesktop', # 0x230
    'NtUserSetThreadLayoutHandles', # 0x231
    'NtUserSetThreadState', # 0x232
    'NtUserSetTimer', # 0x233
    'NtUserSetProcessDPIAware', # 0x234
    'NtUserSetWindowFNID', # 0x235
    'NtUserSetWindowLong', # 0x236
    'NtUserSetWindowPlacement', # 0x237
    'NtUserSetWindowPos', # 0x238
    'NtUserSetWindowRgn', # 0x239
    'NtUserGetWindowRgnEx', # 0x23a
    'NtUserSetWindowRgnEx', # 0x23b
    'NtUserSetWindowsHookAW', # 0x23c
    'NtUserSetWindowsHookEx', # 0x23d
    'NtUserSetWindowStationUser', # 0x23e
    'NtUserSetWindowWord', # 0x23f
    'NtUserSetWinEventHook', # 0x240
    'NtUserShowCaret', # 0x241
    'NtUserShowScrollBar', # 0x242
    'NtUserShowWindow', # 0x243
    'NtUserShowWindowAsync', # 0x244
    'NtUserSoundSentry', # 0x245
    'NtUserSwitchDesktop', # 0x246
    'NtUserSystemParametersInfo', # 0x247
    'NtUserTestForInteractiveUser', # 0x248
    'NtUserThunkedMenuInfo', # 0x249
    'NtUserThunkedMenuItemInfo', # 0x24a
    'NtUserToUnicodeEx', # 0x24b
    'NtUserTrackMouseEvent', # 0x24c
    'NtUserTrackPopupMenuEx', # 0x24d
    'NtUserCalcMenuBar', # 0x24e
    'NtUserPaintMenuBar', # 0x24f
    'NtUserTranslateAccelerator', # 0x250
    'NtUserTranslateMessage', # 0x251
    'NtUserUnhookWindowsHookEx', # 0x252
    'NtUserUnhookWinEvent', # 0x253
    'NtUserUnloadKeyboardLayout', # 0x254
    'NtUserUnlockWindowStation', # 0x255
    'NtUserUnregisterClass', # 0x256
    'NtUserUnregisterUserApiHook', # 0x257
    'NtUserUnregisterHotKey', # 0x258
    'NtUserUpdateInputContext', # 0x259
    'NtUserUpdateInstance', # 0x25a
    'NtUserUpdateLayeredWindow', # 0x25b
    'NtUserGetLayeredWindowAttributes', # 0x25c
    'NtUserSetLayeredWindowAttributes', # 0x25d
    'NtUserUpdatePerUserSystemParameters', # 0x25e
    'NtUserUserHandleGrantAccess', # 0x25f
    'NtUserValidateHandleSecure', # 0x260
    'NtUserValidateRect', # 0x261
    'NtUserValidateTimerCallback', # 0x262
    'NtUserVkKeyScanEx', # 0x263
    'NtUserWaitForInputIdle', # 0x264
    'NtUserWaitForMsgAndEvent', # 0x265
    'NtUserWaitMessage', # 0x266
    'NtUserWin32PoolAllocationStats', # 0x267
    'NtUserWindowFromPhysicalPoint', # 0x268
    'NtUserWindowFromPoint', # 0x269
    'NtUserYieldTask', # 0x26a
    'NtUserRemoteConnect', # 0x26b
    'NtUserRemoteRedrawRectangle', # 0x26c
    'NtUserRemoteRedrawScreen', # 0x26d
    'NtUserRemoteStopScreenUpdates', # 0x26e
    'NtUserCtxDisplayIOCtl', # 0x26f
    'NtUserRegisterSessionPort', # 0x270
    'NtUserUnregisterSessionPort', # 0x271
    'NtUserUpdateWindowTransform', # 0x272
    'NtUserDwmStartRedirection', # 0x273
    'NtUserDwmStopRedirection', # 0x274
    'NtUserDwmHintDxUpdate', # 0x275
    'NtUserDwmGetDxRgn', # 0x276
    'NtUserGetWindowMinimizeRect', # 0x277
    'NtGdiEngAssociateSurface', # 0x278
    'NtGdiEngCreateBitmap', # 0x279
    'NtGdiEngCreateDeviceSurface', # 0x27a
    'NtGdiEngCreateDeviceBitmap', # 0x27b
    'NtGdiEngCreatePalette', # 0x27c
    'NtGdiEngComputeGlyphSet', # 0x27d
    'NtGdiEngCopyBits', # 0x27e
    'NtGdiEngDeletePalette', # 0x27f
    'NtGdiEngDeleteSurface', # 0x280
    'NtGdiEngEraseSurface', # 0x281
    'NtGdiEngUnlockSurface', # 0x282
    'NtGdiEngLockSurface', # 0x283
    'NtGdiEngBitBlt', # 0x284
    'NtGdiEngStretchBlt', # 0x285
    'NtGdiEngPlgBlt', # 0x286
    'NtGdiEngMarkBandingSurface', # 0x287
    'NtGdiEngStrokePath', # 0x288
    'NtGdiEngFillPath', # 0x289
    'NtGdiEngStrokeAndFillPath', # 0x28a
    'NtGdiEngPaint', # 0x28b
    'NtGdiEngLineTo', # 0x28c
    'NtGdiEngAlphaBlend', # 0x28d
    'NtGdiEngGradientFill', # 0x28e
    'NtGdiEngTransparentBlt', # 0x28f
    'NtGdiEngTextOut', # 0x290
    'NtGdiEngStretchBltROP', # 0x291
    'NtGdiXLATEOBJ_cGetPalette', # 0x292
    'NtGdiXLATEOBJ_iXlate', # 0x293
    'NtGdiXLATEOBJ_hGetColorTransform', # 0x294
    'NtGdiCLIPOBJ_bEnum', # 0x295
    'NtGdiCLIPOBJ_cEnumStart', # 0x296
    'NtGdiCLIPOBJ_ppoGetPath', # 0x297
    'NtGdiEngDeletePath', # 0x298
    'NtGdiEngCreateClip', # 0x299
    'NtGdiEngDeleteClip', # 0x29a
    'NtGdiBRUSHOBJ_ulGetBrushColor', # 0x29b
    'NtGdiBRUSHOBJ_pvAllocRbrush', # 0x29c
    'NtGdiBRUSHOBJ_pvGetRbrush', # 0x29d
    'NtGdiBRUSHOBJ_hGetColorTransform', # 0x29e
    'NtGdiXFORMOBJ_bApplyXform', # 0x29f
    'NtGdiXFORMOBJ_iGetXform', # 0x2a0
    'NtGdiFONTOBJ_vGetInfo', # 0x2a1
    'NtGdiFONTOBJ_pxoGetXform', # 0x2a2
    'NtGdiFONTOBJ_cGetGlyphs', # 0x2a3
    'NtGdiFONTOBJ_pifi', # 0x2a4
    'NtGdiFONTOBJ_pfdg', # 0x2a5
    'NtGdiFONTOBJ_pQueryGlyphAttrs', # 0x2a6
    'NtGdiFONTOBJ_pvTrueTypeFontFile', # 0x2a7
    'NtGdiFONTOBJ_cGetAllGlyphHandles', # 0x2a8
    'NtGdiSTROBJ_bEnum', # 0x2a9
    'NtGdiSTROBJ_bEnumPositionsOnly', # 0x2aa
    'NtGdiSTROBJ_bGetAdvanceWidths', # 0x2ab
    'NtGdiSTROBJ_vEnumStart', # 0x2ac
    'NtGdiSTROBJ_dwGetCodePage', # 0x2ad
    'NtGdiPATHOBJ_vGetBounds', # 0x2ae
    'NtGdiPATHOBJ_bEnum', # 0x2af
    'NtGdiPATHOBJ_vEnumStart', # 0x2b0
    'NtGdiPATHOBJ_vEnumStartClipLines', # 0x2b1
    'NtGdiPATHOBJ_bEnumClipLines', # 0x2b2
    'NtGdiGetDhpdev', # 0x2b3
    'NtGdiEngCheckAbort', # 0x2b4
    'NtGdiHT_Get8BPPFormatPalette', # 0x2b5
    'NtGdiHT_Get8BPPMaskPalette', # 0x2b6
    'NtGdiUpdateTransform', # 0x2b7
    'NtGdiSetPUMPDOBJ', # 0x2b8
    'NtGdiBRUSHOBJ_DeleteRbrush', # 0x2b9
    'NtGdiUMPDEngFreeUserMem', # 0x2ba
    'NtGdiDrawStream', # 0x2bb
    'NtGdiDwmGetDirtyRgn', # 0x2bc
    'NtGdiDwmGetSurfaceData', # 0x2bd
    'NtGdiDdDDICreateAllocation', # 0x2be
    'NtGdiDdDDIQueryResourceInfo', # 0x2bf
    'NtGdiDdDDIOpenResource', # 0x2c0
    'NtGdiDdDDIDestroyAllocation', # 0x2c1
    'NtGdiDdDDISetAllocationPriority', # 0x2c2
    'NtGdiDdDDIQueryAllocationResidency', # 0x2c3
    'NtGdiDdDDICreateDevice', # 0x2c4
    'NtGdiDdDDIDestroyDevice', # 0x2c5
    'NtGdiDdDDICreateContext', # 0x2c6
    'NtGdiDdDDIDestroyContext', # 0x2c7
    'NtGdiDdDDICreateSynchronizationObject', # 0x2c8
    'NtGdiDdDDIDestroySynchronizationObject', # 0x2c9
    'NtGdiDdDDIWaitForSynchronizationObject', # 0x2ca
    'NtGdiDdDDISignalSynchronizationObject', # 0x2cb
    'NtGdiDdDDIGetRuntimeData', # 0x2cc
    'NtGdiDdDDIQueryAdapterInfo', # 0x2cd
    'NtGdiDdDDILock', # 0x2ce
    'NtGdiDdDDIUnlock', # 0x2cf
    'NtGdiDdDDIGetDisplayModeList', # 0x2d0
    'NtGdiDdDDISetDisplayMode', # 0x2d1
    'NtGdiDdDDIGetMultisampleMethodList', # 0x2d2
    'NtGdiDdDDIPresent', # 0x2d3
    'NtGdiDdDDIRender', # 0x2d4
    'NtGdiDdDDIOpenAdapterFromDeviceName', # 0x2d5
    'NtGdiDdDDIOpenAdapterFromHdc', # 0x2d6
    'NtGdiDdDDICloseAdapter', # 0x2d7
    'NtGdiDdDDIGetSharedPrimaryHandle', # 0x2d8
    'NtGdiDdDDIEscape', # 0x2d9
    'NtGdiDdDDIQueryStatistics', # 0x2da
    'NtGdiDdDDISetVidPnSourceOwner', # 0x2db
    'NtGdiDdDDIGetPresentHistory', # 0x2dc
    'NtGdiDdDDICreateOverlay', # 0x2dd
    'NtGdiDdDDIUpdateOverlay', # 0x2de
    'NtGdiDdDDIFlipOverlay', # 0x2df
    'NtGdiDdDDIDestroyOverlay', # 0x2e0
    'NtGdiDdDDIWaitForVerticalBlankEvent', # 0x2e1
    'NtGdiDdDDISetGammaRamp', # 0x2e2
    'NtGdiDdDDIGetDeviceState', # 0x2e3
    'NtGdiDdDDICreateDCFromMemory', # 0x2e4
    'NtGdiDdDDIDestroyDCFromMemory', # 0x2e5
    'NtGdiDdDDISetContextSchedulingPriority', # 0x2e6
    'NtGdiDdDDIGetContextSchedulingPriority', # 0x2e7
    'NtGdiDdDDISetProcessSchedulingPriorityClass', # 0x2e8
    'NtGdiDdDDIGetProcessSchedulingPriorityClass', # 0x2e9
    'NtGdiDdDDIReleaseProcessVidPnSourceOwners', # 0x2ea
    'NtGdiDdDDIGetScanLine', # 0x2eb
    'NtGdiDdDDISetQueuedLimit', # 0x2ec
    'NtGdiDdDDIPollDisplayChildren', # 0x2ed
    'NtGdiDdDDIInvalidateActiveVidPn', # 0x2ee
    'NtGdiDdDDICheckOcclusion', # 0x2ef
    'NtGdiDdDDIWaitForIdle', # 0x2f0
    'NtGdiDdDDICheckMonitorPowerState', # 0x2f1
    'NtGdiDdDDICheckExclusiveOwnership', # 0x2f2
    'NtGdiDdDDISetDisplayPrivateDriverFormat', # 0x2f3
    'NtGdiDdDDISharedPrimaryLockNotification', # 0x2f4
    'NtGdiDdDDISharedPrimaryUnLockNotification', # 0x2f5
    'DxgStubEnableDirectDrawRedirection', # 0x2f6
    'DxgStubDeleteDirectDrawObject', # 0x2f7
    'NtGdiGetNumberOfPhysicalMonitors', # 0x2f8
    'NtGdiGetPhysicalMonitors', # 0x2f9
    'NtGdiGetPhysicalMonitorDescription', # 0x2fa
    'NtGdiDestroyPhysicalMonitor', # 0x2fb
    'NtGdiDDCCIGetVCPFeature', # 0x2fc
    'NtGdiDDCCISetVCPFeature', # 0x2fd
    'NtGdiDDCCISaveCurrentSettings', # 0x2fe
    'NtGdiDDCCIGetCapabilitiesStringLength', # 0x2ff
    'NtGdiDDCCIGetCapabilitiesString', # 0x300
    'NtGdiDDCCIGetTimingReport', # 0x301
    'NtUserSetMirrorRendering', # 0x302
    'NtUserShowSystemCursor', # 0x303
    ],
]



# Volatility
# Copyright (C) 2007-2013 Volatility Foundation
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License Version 2 as
# published by the Free Software Foundation.  You may not use, modify or
# distribute this program under any other version of the GNU General
# Public License.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

"""
@author:       Andrew Case
@license:      GNU General Public License 2.0
@contact:      atcuno@gmail.com
@organization: 
"""

import os
import volatility.obj as obj
import volatility.debug as debug
import volatility.plugins.mac.proc_maps as proc_maps

class mac_dump_maps(proc_maps.mac_proc_maps):
    """ Dumps memory ranges of processes """

    def __init__(self, config, *args, **kwargs):         
        proc_maps.mac_proc_maps.__init__(self, config, *args, **kwargs)         
        self._config.add_option('MAP_ADDRESS', short_option = 's', default = None, help = 'Filter by starting address of map', action = 'store', type = 'long') 
        self._config.add_option('OUTPUTFILE', short_option = 'O', default = None, help = 'Output File', action = 'store', type = 'str')
    
    def render_text(self, outfd, data):
        if not self._config.OUTPUTFILE:
            debug.error("Please specify an OUTPUTFILE")
        elif os.path.exists(self._config.OUTPUTFILE):
            debug.error("Cowardly refusing to overwrite an existing file")
                    
        outfile = open(self._config.OUTPUTFILE, "wb+")
        map_address = self._config.MAP_ADDRESS

        size = 0
        for proc, map in data:
            if not map_address or map_address == map.links.start:
                for page in self._read_addr_range(proc, map.links.start, map.links.end):
                    size += len(page)
                    outfile.write(page)
        
        outfile.close()
        outfd.write("Wrote {0} bytes\n".format(size))

    def _read_addr_range(self, proc, start, end):
        pagesize = 4096 

        # set the as with our new dtb so we can read from userland
        proc_as = proc.get_process_address_space()

        while start < end:
            page = proc_as.zread(start, pagesize)
            yield page
            start = start + pagesize
